// 04/27/2004 - Fixed some bugs
// 10/21/2004 - Added support for changing flats
// 03/12/2006 - Reorganized, added format independent classes
// 07/07/2006 - Added support for doors openable by guards
// 07/09/2006 - Added prefabricated brown barrel

#include <windows.h>
#include <stdio.h>
#pragma comment(lib,"user32.lib") // TJL - Added to allow compile in VC++ 2005 Express (worked for me, at least)

#include "Prefabs.h"
#include "MapThing.h"
#include "MapLinedef.h"
#include "MapLine.h"

CMapConvert::CMapConvert ()
{
	m_iConversionType = CTYPE_INVALID;

	m_lpList = NULL;
	m_lpVertices = NULL;
	m_lpSectors = NULL;
	m_lpThings = NULL;
	m_iNextSector = 0;
	m_lpAssign = NULL;
	m_lpSectorTable = new USHORT[4096];

	lstrcpy(m_szCeiling,"FLAT19");
	lstrcpy(m_szFloor,"FLOOR7_1");

	m_iVertStrip = 0;
	m_iHorzStrip = 0;
	m_xNextDoor = -2016;
	m_yNextDoor = 2016;
	m_iNextPoly = 1;
	m_iNextSecret = 1;
	m_lpQueue = NULL;

	m_iLightLevel = 160;

	m_bSimpleConversion = TRUE;
	m_bUseAutoDoors = TRUE;
	m_bUseUpperPanels = TRUE;
	m_bBuildDoorSockets = TRUE;
	m_bAllowFloorZero = TRUE;
	m_iSecretDoorType = SECRET_SIMPLE;
}

CMapConvert::~CMapConvert ()
{
	ResetConversion();

	delete m_lpSectorTable;
	m_lpSectorTable = NULL;
}

VOID CMapConvert::ResetConversion (VOID)
{
	LPLINE_LIST lpLine;
	LPSECTOR_LIST lpSector;
	LPVERTEX_LIST lpVertex;
	LPTHING_LIST lpThing;
	while(m_lpList)
	{
		lpLine = m_lpList;
		m_lpList = m_lpList->Next;
		delete lpLine->lpLine;
		delete lpLine;
	}
	while(m_lpSectors)
	{
		lpSector = m_lpSectors;
		m_lpSectors = m_lpSectors->Next;
		delete lpSector;
	}
	while(m_lpVertices)
	{
		lpVertex = m_lpVertices;
		m_lpVertices = m_lpVertices->Next;
		delete lpVertex;
	}
	while(m_lpThings)
	{
		lpThing = m_lpThings;
		m_lpThings = m_lpThings->Next;
		delete lpThing->lpThing;
		delete lpThing;
	}

	m_iNextSector = 0;

	ZeroMemory(m_lpSectorTable,4096 * sizeof(USHORT));

	m_iVertStrip = 0;
	m_iHorzStrip = 0;
	m_xNextDoor = -2016;
	m_yNextDoor = 2016;
	m_iNextPoly = 1;
	m_iNextSecret = 1;
	m_lpQueue = NULL;
}

INT CMapConvert::GetConversionType (VOID)
{
	return m_iConversionType;
}

VOID CMapConvert::SetConversionType (INT iConversionType)
{
	if(iConversionType == CTYPE_DOOM || iConversionType == CTYPE_ZDOOM)
		m_iConversionType = iConversionType;
	else if (iConversionType == CTYPE_ZDOOMTC || iConversionType == CTYPE_BSTC)
	{
		m_iConversionType = iConversionType;

		lstrcpy(m_szCeiling,"CEILING");
		lstrcpy(m_szFloor,"FLOOR");

		if(m_iConversionType == CTYPE_BSTC)
			m_iLightLevel = 224;
		else
			m_iLightLevel = 255;

		m_bSimpleConversion = FALSE;
		m_bUseAutoDoors = TRUE;
		m_bUseUpperPanels = FALSE;
		m_bBuildDoorSockets = TRUE;
		m_bAllowFloorZero = TRUE;
		m_iSecretDoorType = SECRET_GL;
	}
	else if (iConversionType == CTYPE_EDGE)
	{
		m_iConversionType = iConversionType;

		lstrcpy(m_szCeiling,"CEILING");
		lstrcpy(m_szFloor,"FLOOR");

		m_iLightLevel = 255;

		m_bSimpleConversion = FALSE;
		m_bUseAutoDoors = FALSE;
		m_bUseUpperPanels = FALSE;
		m_bBuildDoorSockets = FALSE;
		m_bAllowFloorZero = TRUE;
		m_iSecretDoorType = SECRET_STAY_OPEN;
	}
}

VOID CMapConvert::UseAutoDoors (BOOL bUseAutoDoors)
{
	m_bUseAutoDoors = bUseAutoDoors;
}

VOID CMapConvert::AllowFloorZero (BOOL bAllowFloorZero)
{
	m_bAllowFloorZero = bAllowFloorZero;
}

VOID CMapConvert::SetLightLevel (INT iLightLevel)
{
	if(m_iLightLevel >= 0)
		m_iLightLevel = iLightLevel;
}

VOID CMapConvert::ChangeFlats (LPSTR lpszCeiling, LPSTR lpszFloor)
{
	if(lpszCeiling && *lpszCeiling)
		lstrcpy(m_szCeiling,lpszCeiling);
	if(lpszFloor && *lpszFloor)
		lstrcpy(m_szFloor,lpszFloor);
}

VOID CMapConvert::GetFlats (LPSTR lpszCeiling, LPSTR lpszFloor)
{
	if(lpszCeiling)
		lstrcpy(lpszCeiling,m_szCeiling);
	if(lpszFloor)
		lstrcpy(lpszFloor,m_szFloor);
}

BOOL CMapConvert::RunConversion (LPSTR lpszLevel, LPSTR lpszOutput, USHORT* lpFloor, USHORT* lpObject, BOOL bDecorations)
{
	BOOL bSuccess = TRUE;
	try
	{
		CPrefabs::SetLightLevel(m_iLightLevel);
		CPrefabs::SetFlats(m_szFloor,m_szCeiling);
		CPrefabs::SetUpperPanels(m_bUseUpperPanels);
		BuildLineStructure(lpFloor,lpObject);
		BuildSecretDoors(lpFloor,lpObject);
		BuildSectors(lpFloor);
		BuildWalkover(lpFloor,lpObject);
		ConvertFinale(lpObject);
		if(m_iConversionType == CTYPE_BSTC)
			BuildUsableWalls(lpFloor, lpObject);
		MergeLines();
		if(m_bSimpleConversion)
		{
			BuildOutsideViews();
			BuildPrefabs(lpObject);
		}
		else
		{
			if(m_iConversionType != CTYPE_DOOM && m_iConversionType != CTYPE_EDGE)
				BuildHoldingArea();
			BuildDMPlayerStarts(lpObject,lpFloor);
			BuildPlayerStarts(lpObject,lpFloor);
		}
		BuildThings(lpObject,lpFloor);
		if(m_iConversionType != CTYPE_DOOM && m_iConversionType != CTYPE_EDGE)
			BuildPolys();
		FixSidedefs();
		BuildVertexList();
		if(m_lpList)
			BuildWadFile(lpszLevel,lpszOutput,bDecorations);
		else
			bSuccess = FALSE;
	}
	catch(...)
	{
		bSuccess = FALSE;
	}
	return bSuccess;
}

SHORT CMapConvert::GetNewSector (VOID)
{
	return m_iNextSector++;
}

//

CMapLine* CMapConvert::FindLine (LPVERTEX lpvFrom, LPVERTEX lpvTo)
{
	LPLINE_LIST lpList = m_lpList;
	CMapLine* lpLine;
	while(lpList)
	{
		lpLine = lpList->lpLine;
		if(lpLine->m_vFrom.x == lpvFrom->x && lpLine->m_vFrom.y == lpvFrom->y)
		{
			if(lpLine->m_vTo.x == lpvTo->x && lpLine->m_vTo.y == lpvTo->y)
				return lpLine;
		}
		//TJL
		if(lpLine->m_vTo.x == lpvFrom->x && lpLine->m_vTo.y == lpvFrom->y)
		{
			if(lpLine->m_vFrom.x == lpvTo->x && lpLine->m_vFrom.y == lpvTo->y)
				return lpLine;
		}
		//
		lpList = lpList->Next;
	}
	return NULL;
}

LPLINE_LIST CMapConvert::FindNext (LPVERTEX lpvFrom, LPLINE_LIST* lplpPrev)
{
	LPLINE_LIST lpList = m_lpList, lpPrev = NULL;
	LPVERTEX lpv;
	while(lpList)
	{
		lpv = &lpList->lpLine->m_vFrom;
		if(lpv->x == lpvFrom->x && lpv->y == lpvFrom->y)
			break;
		lpPrev = lpList;
		lpList = lpList->Next;
	}
	*lplpPrev = lpPrev;
	return lpList;
}

USHORT CMapConvert::FindVertex (LPVERTEX lpVertex)
{
	LPVERTEX_LIST lpList = m_lpVertices;
	USHORT v = 0;
	while(lpList)
	{
		if(lpList->Vertex.x == lpVertex->x && lpList->Vertex.y == lpVertex->y)
			return v;
		v++;
		lpList = lpList->Next;
	}
	return 65535;
}

USHORT CMapConvert::FindSector (INT x, INT y)
{
	USHORT iSector = m_lpSectorTable[y * 64 + x];
	return iSector;
}

VOID CMapConvert::SetFrontDoor (INT iType, LPSTR lpszTexture, INT bdir)
{
	LPSTR lpszSet = "-";
	switch(iType)
	{
	case DOOR_UNLOCKED:
		if (bdir)
			lpszSet = "NZUD1";
		else
			lpszSet = "NZUD12";
		break;
	case DOOR_SILVER:
	case DOOR_GOLD:
		if (bdir)
			lpszSet = "NZLD1";
		else
			lpszSet = "NZLD12";
		break;
	case DOOR_ELEVATOR:
		if (bdir)
			lpszSet = "NZED1";
		else
			lpszSet = "NZED12";
		break;
	}
	lstrcpy(lpszTexture,lpszSet);
}

VOID CMapConvert::SetBackDoor (INT iType, LPSTR lpszTexture, INT bdir)
{
	LPSTR lpszSet = "-";
	switch(iType)
	{
	case DOOR_UNLOCKED:
		if (bdir)
			lpszSet = "NZUD2";
		else
			lpszSet = "NZUD22";
		break;
	case DOOR_SILVER:
	case DOOR_GOLD:
		if (bdir)
			lpszSet = "NZLD2";
		else
			lpszSet = "NZLD22";
		break;
	case DOOR_ELEVATOR:
		if (bdir)
			lpszSet = "NZED2";
		else
			lpszSet = "NZED22";
		break;
	}
	lstrcpy(lpszTexture,lpszSet);
}

VOID CMapConvert::PositionLine (CMapLine* lpLine, INT x, INT y, INT iDir)
{
	switch(iDir)
	{
	case LINE_LEFT:
		lpLine->m_vFrom.x = x * 64 - 2048;
		lpLine->m_vFrom.y = 2048 - (y + 1) * 64;
		lpLine->m_vTo.x = lpLine->m_vFrom.x;
		lpLine->m_vTo.y = lpLine->m_vFrom.y + 64;
		break;
	case LINE_RIGHT:
		lpLine->m_vFrom.x = (x + 1) * 64 - 2048;
		lpLine->m_vFrom.y = 2048 - y * 64;
		lpLine->m_vTo.x = lpLine->m_vFrom.x;
		lpLine->m_vTo.y = lpLine->m_vFrom.y - 64;
		break;
	case LINE_ABOVE:
		lpLine->m_vFrom.x = x * 64 - 2048;
		lpLine->m_vFrom.y = 2048 - y * 64;
		lpLine->m_vTo.x = lpLine->m_vFrom.x + 64;
		lpLine->m_vTo.y = lpLine->m_vFrom.y;
		break;
	case LINE_BELOW:
		lpLine->m_vFrom.x = (x + 1) * 64 - 2048;
		lpLine->m_vFrom.y = 2048 - (y + 1) * 64;
		lpLine->m_vTo.x = lpLine->m_vFrom.x - 64;
		lpLine->m_vTo.y = lpLine->m_vFrom.y;
		break;
	}
}

VOID CMapConvert::PositionThing (CMapThing* lpThing, INT x, INT y)
{
	lpThing->m_x = (x * 64 - 2048) + 32;
	lpThing->m_y = (2048 - y * 64) - 32;
}

VOID CMapConvert::FlipLinedef (CMapLine* lpLine, BOOL bAll)
{
	VERTEX vTemp;
	CopyMemory(&vTemp,&lpLine->m_vFrom,sizeof(VERTEX));
	CopyMemory(&lpLine->m_vFrom,&lpLine->m_vTo,sizeof(VERTEX));
	CopyMemory(&lpLine->m_vTo,&vTemp,sizeof(VERTEX));
	if(bAll)
	{
		SIDEDEF sTemp;
		CopyMemory(&sTemp,&lpLine->m_sRight,sizeof(SIDEDEF));
		CopyMemory(&lpLine->m_sRight,&lpLine->m_sLeft,sizeof(SIDEDEF));
		CopyMemory(&lpLine->m_sLeft,&sTemp,sizeof(SIDEDEF));
	}
}

VOID CMapConvert::FillSector (LPSECTOR lpSector)
{
	lpSector->Floor = 0;
	lpSector->Ceiling = (m_bUseUpperPanels) ? 80 : 64;
	lstrcpy(lpSector->szFloor,m_szFloor);
	lstrcpy(lpSector->szCeiling,m_szCeiling);
	lpSector->Light = m_iLightLevel;
	lpSector->Special = 0;
	lpSector->Tag = 0;
}

VOID CMapConvert::AddLine (CMapLine* lpLine)
{
	LPLINE_LIST lpNew = new LINE_LIST;
	lpNew->lpLine = new CMapLine(lpLine);
	lpNew->Next = m_lpList;
	m_lpList = lpNew;
}

VOID CMapConvert::AddImpassible (CMapLine* lpLine, USHORT iTexture, BOOL bSecretElevator)
{
	lpLine->m_Flags = 1;			// Impassible

	if(iTexture < 16)
	{
		if (lpLine->m_vFrom.y == lpLine->m_vTo.y)
			wsprintf(lpLine->m_sRight.szMiddle,"NZ0%X",iTexture);
		else
			wsprintf(lpLine->m_sRight.szMiddle,"NZ0%X2",iTexture);
	}
	else
	{
		if(iTexture == 21 && lpLine->m_vFrom.x == lpLine->m_vTo.x)
		{
			if(m_iConversionType == CTYPE_ZDOOM)
			{
				// TJL - Elevator Switch
				lpLine->m_Flags |= 1024;
				lpLine->m_Special = 74;
				if(bSecretElevator)
					lpLine->m_Args[0] = 10;
				else
					lpLine->m_Args[0] = 0;
				lpLine->m_Args[1] = 0;
				lpLine->m_Args[2] = 0;
				lpLine->m_Args[3] = 0;
				lpLine->m_Args[4] = 0;
			}
			else if(m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
			{
				lpLine->m_Flags |= 1024;
				lpLine->m_Special = 226;
				lpLine->m_Args[0] = 10;
				lpLine->m_Args[1] = 0;
				if(bSecretElevator)
					lpLine->m_Args[2] = 10;
				else
					lpLine->m_Args[2] = 0;
				lpLine->m_Args[3] = 0;
				lpLine->m_Args[4] = 0;
			}
			else if(m_iConversionType == CTYPE_DOOM || m_iConversionType == CTYPE_EDGE)
			{
				if(bSecretElevator)
					lpLine->m_Types = 51;
				else
					lpLine->m_Types = 11;
			}

			if(m_iConversionType == CTYPE_BSTC)
				lstrcpy(lpLine->m_sRight.szMiddle,"NZ152");
			else
				lstrcpy(lpLine->m_sRight.szMiddle,"NZESW1");
		}
		else
		{
			if (lpLine->m_vFrom.y == lpLine->m_vTo.y)
				wsprintf(lpLine->m_sRight.szMiddle,"NZ%X",iTexture);
			else
				wsprintf(lpLine->m_sRight.szMiddle,"NZ%X2",iTexture);
		}
	}
	lstrcpy(lpLine->m_sRight.szLower,"-");
	lstrcpy(lpLine->m_sRight.szUpper,"-");
	lpLine->m_sRight.Sector = -1;	// Sorted Later
	AddLine(lpLine);
}

VOID CMapConvert::AddSidedLine (CMapLine* lpLine, USHORT iSector, USHORT iTexture)
{
	CMapLine* lpFind = FindLine(&lpLine->m_vTo,&lpLine->m_vFrom);
	lpLine->m_sRight.Sector = iSector;
	if(lpFind)
	{
		lpFind->m_Flags = 4;		// Two Sided
		lstrcpy(lpLine->m_sRight.szMiddle,"-");
		CopyMemory(&lpFind->m_sLeft,&lpLine->m_sRight,sizeof(SIDEDEF));
		lstrcpy(lpFind->m_sRight.szMiddle,"-");
		lstrcpy(lpFind->m_sLeft.szLower,"-");
		lstrcpy(lpFind->m_sLeft.szUpper,"-");
	}
	else
	{
		lpLine->m_Flags = 1;		// Impassible
		if(iTexture < 16)
		{
			if (lpLine->m_vFrom.y == lpLine->m_vTo.y)
				wsprintf(lpLine->m_sRight.szMiddle,"NZ0%X",iTexture);
			else
				wsprintf(lpLine->m_sRight.szMiddle,"NZ0%X2",iTexture);
		}
		else
		{
			if (lpLine->m_vFrom.y == lpLine->m_vTo.y)
				wsprintf(lpLine->m_sRight.szMiddle,"NZ%X",iTexture);
			else
				wsprintf(lpLine->m_sRight.szMiddle,"NZ%X2",iTexture);
		}
		AddLine(lpLine);
	}
}

VOID CMapConvert::AddSecretLine (CMapLine* lpLine, USHORT iSector, USHORT iTexture, INT DoorNumber, BOOL AdjacentToDoor)
{
	CMapLine* lpFind = FindLine(&lpLine->m_vTo,&lpLine->m_vFrom);
	lpLine->m_sRight.Sector = iSector;

	if(lpFind)
	{
		if(lpFind->m_sRight.Sector == -1 && AdjacentToDoor)
		{
			lpFind->m_sRight.Sector = iSector;
			FlipLinedef(lpFind);
		}
		lpFind->m_Flags = 4;		// Two Sided
		lstrcpy(lpLine->m_sRight.szMiddle,"-");
		CopyMemory(&lpFind->m_sLeft,&lpLine->m_sRight,sizeof(SIDEDEF));
		lstrcpy(lpFind->m_sRight.szMiddle,"-");
		lstrcpy(lpFind->m_sLeft.szLower,"-");
		lstrcpy(lpFind->m_sLeft.szUpper,"-");
		if(iTexture < 16)
		{
			if (lpFind->m_vFrom.y == lpFind->m_vTo.y)
				wsprintf(lpFind->m_sRight.szUpper,"NZ0%X",iTexture);
			else
				wsprintf(lpFind->m_sRight.szUpper,"NZ0%X2",iTexture);
		}
		else
		{
			if (lpFind->m_vFrom.y == lpFind->m_vTo.y)
				wsprintf(lpFind->m_sRight.szUpper,"NZ%X",iTexture);
			else
				wsprintf(lpFind->m_sRight.szUpper,"NZ%X2",iTexture);
		}
		if(m_iConversionType == CTYPE_DOOM || m_iConversionType == CTYPE_EDGE)
		{
			if(m_iSecretDoorType == SECRET_STAY_OPEN)
				lpFind->m_Types = 103;
			else
				lpFind->m_Types = 1;
		}
		else if(m_iConversionType == CTYPE_ZDOOM || m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
		{
			if(m_iSecretDoorType == SECRET_SIMPLE)
			{
				lpFind->m_Flags |= (512 + 1024);
				lpFind->m_Special = 202;
				lpFind->m_Args[0] = 0;
				lpFind->m_Args[1] = 16;
				lpFind->m_Args[2] = 0;
				lpFind->m_Args[3] = 75;
				lpFind->m_Args[4] = 0;
			}
			else if(m_iSecretDoorType == SECRET_STAY_OPEN)
			{
				lpFind->m_Flags |= (32 + 1024);
				lpFind->m_Special = 41;
				lpFind->m_Args[0] = 0;
				lpFind->m_Args[1] = 4;
				lpFind->m_Args[2] = 64;
				lpFind->m_Args[3] = 0;
				lpFind->m_Args[4] = 0;
			}
			else if(m_iSecretDoorType == SECRET_GL)
			{
				lpFind->m_Flags |= (32 + 1024);
				lpFind->m_Special = 226;
				lpFind->m_Args[0] = 15;
				lpFind->m_Args[1] = 0;
				lpFind->m_Args[2] = DoorNumber - 800;
				lpFind->m_Args[3] = 0;
				lpFind->m_Args[4] = 0;
			}
		}
	}
	else
	{
		lpLine->m_Flags = 17;		// Impassible & Lower unpegged
		if(iTexture < 16)
		{
			if (lpLine->m_vFrom.y == lpLine->m_vTo.y)
				wsprintf(lpLine->m_sRight.szMiddle,"NZ0%X",iTexture);
			else
				wsprintf(lpLine->m_sRight.szMiddle,"NZ0%X2",iTexture);
		}
		else
		{
			if (lpLine->m_vFrom.y == lpLine->m_vTo.y)
				wsprintf(lpLine->m_sRight.szMiddle,"NZ%X",iTexture);
			else
				wsprintf(lpLine->m_sRight.szMiddle,"NZ%X2",iTexture);
		}
		lstrcpy(lpLine->m_sRight.szLower,"-");
		lstrcpy(lpLine->m_sRight.szUpper,"-");
		AddLine(lpLine);
	}
}

VOID CMapConvert::AddSector (LPSECTOR lpSector)
{
	LPSECTOR_LIST lpNew = new SECTOR_LIST;
	CopyMemory(&lpNew->Sector,lpSector,sizeof(SECTOR));
	lpNew->Next = NULL;
	if(m_lpSectors)
	{
		LPSECTOR_LIST lpList = m_lpSectors;
		while(lpList->Next)
			lpList = lpList->Next;
		lpList->Next = lpNew;
	}
	else
		m_lpSectors = lpNew;
}

VOID CMapConvert::AddThing (CMapThing* lpThing)
{
	LPTHING_LIST lpNew = new THING_LIST;
	lpNew->lpThing = new CMapThing(lpThing);
	lpNew->Next = NULL;
	if(m_lpThings)
	{
		LPTHING_LIST lpList = m_lpThings;
		while(lpList->Next)
			lpList = lpList->Next;
		lpList->Next = lpNew;
	}
	else
		m_lpThings = lpNew;
}

VOID CMapConvert::RemoveLine (CMapLine* lpLine)
{
	LPLINE_LIST lpList, lpPrev = NULL;
	lpList = m_lpList;
	while(lpList)
	{
		if(lpList->lpLine == lpLine)
		{
			if(lpPrev)
				lpPrev = lpList->Next;
			else
				m_lpList = lpList->Next;
			delete lpList->lpLine;
			delete lpList;
			break;
		}
		lpPrev = lpList;
		lpList = lpList->Next;
	}
}

VOID CMapConvert::BuildLineStructure (USHORT* lpFloor, USHORT* lpObject)
{
	INT x, y; //, iObject = 0, switchedx, switchedy, parameter = 0;
	USHORT f, b, o, *lpRow = lpFloor, *lpObj = lpObject;
	CMapLine Line;

	// There are two ways to handle m_bAllowFloorZero:
	// 1.  Replace all occurrances of floor zero with a valid floor type
	// 2.  Treat floor zero as a separate floor type so it gets its own sector
	// I believe option 2 is correct.

	for(y = 0; y < 64; y++)
	{
		for(x = 0; x < 64; x++)
		{
			f = lpRow[x];
			o = lpObj[x];

			if(f >= 88 && f <= 105)
			{
				INT iType, bDir, bSilence;
				if (m_iConversionType == CTYPE_BSTC)
				{
					switch(o)
					{
					case 55:
						iType = DOOR_RED;
						break;
					case 56:
						iType = DOOR_YELLOW;
						break;
					case 57:
						iType = DOOR_GREEN;
						break;
					case 58:
						iType = DOOR_BLUE;
						break;
					case 59:
						iType = DOOR_GOLDBS;
						break;
					default:
						printf("Other Door Action: %X\n", o);
					case 0:
						iType = DOOR_UNLOCKED;
						break;
					}
					switch (f)
					{
					case 98:
						iType = DOOR_ONEWAYW;
						break;
					case 99:
						iType = DOOR_ONEWAYN;
						break;
					case 100:
						iType = DOOR_ONEWAYE;
						break;
					case 101:
						iType = DOOR_ONEWAYS;
						break;
					}
				}
				else
				{
					switch(f)
					{
					case 90:	// unlocked
					case 91:
						iType = DOOR_UNLOCKED;
						break;
					case 92:	// silver
					case 93:
						iType = DOOR_GOLD;
						break;
					case 94:	// gold
					case 95:
						iType = DOOR_SILVER;
						break;
					case 100:	// elevator
					case 101:
						iType = DOOR_ELEVATOR;
						break;
					default:
						iType = DOOR_UNLOCKED;
						break;
					}
				}
				bSilence = 0;
				if(f & 1)
				{
					bDir = 1;	// North/South
					if (lpRow[(x-64)] != lpRow[(x+64)])
						bSilence = 1;
				}
				else
				{
					bDir = 0;	// East/West
					if (lpRow[x-1] != lpRow[x+1])
						bSilence = 1;
				}
				if (m_iConversionType == CTYPE_BSTC)
					BuildDoorBS(x,y,iType,bDir,bSilence,f);
				else
					BuildDoor(x,y,iType,bDir,bSilence);
			}
			else if(f >= 106 || (m_bAllowFloorZero && f == 0))
			{
				if(x > 0)
				{
					b = lpFloor[y * 64 + (x - 1)];	// Left
					if(b == 0 && m_bAllowFloorZero)
						b = 65535;
				}
				else
					b = 1;							// Left (Bad)
				if(b < 88)
				{
					Line.Reset();
					PositionLine(&Line,x,y,LINE_LEFT);
					AddImpassible(&Line,b,f == 107);
				}

				if(x < 63)
				{
					b = lpFloor[y * 64 + (x + 1)];	// Right
					if(b == 0 && m_bAllowFloorZero)
						b = 65535;
				}
				else
					b = 1;							// Right (Bad)
				if(b < 88)
				{
					Line.Reset();
					PositionLine(&Line,x,y,LINE_RIGHT);
					AddImpassible(&Line,b,f == 107);
				}

				if(y > 0)
				{
					b = lpFloor[(y - 1) * 64 + x];	// Above
					if(b == 0 && m_bAllowFloorZero)
						b = 65535;
				}
				else
					b = 1;							// Above (Bad)
				if(b < 88)
				{
					Line.Reset();
					PositionLine(&Line,x,y,LINE_ABOVE);
					AddImpassible(&Line,b,f == 107);
				}

				if(y < 63)
				{
					b = lpFloor[(y + 1) * 64 + x];	// Below
					if(b == 0 && m_bAllowFloorZero)
						b = 65535;
				}
				else
					b = 1;							// Below (Bad)
				if(b < 88)
				{
					Line.Reset();
					PositionLine(&Line,x,y,LINE_BELOW);
					AddImpassible(&Line,b,f == 107);
				}
			}
		}
		lpRow += 64;
		lpObj += 64;
	}
}

VOID CMapConvert::BuildSecretDoors (USHORT* lpFloor, USHORT* lpObject)
{
	INT x, y, fSides, iDir;
	USHORT b, *lpRow = lpObject, s, f;
	BOOL bDoor;
	CMapLine Line;
	CMapThing Secret;
	SECTOR Sector;
	for(y = 0; y < 64; y++)
	{
		for(x = 0; x < 64; x++)
		{
			if(lpRow[x] == 98 && lpFloor[y*64 + x] <= 85)
			{
				s = m_iNextSector++;
				FillSector(&Sector);
				if (m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
					Sector.Ceiling = 64;
				else
					Sector.Ceiling = 0;

				fSides = 0;
				iDir = -1;

				// TJL - Sector Secret Flag
				if(m_iConversionType == CTYPE_ZDOOM || m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
				{
					Sector.Special = 1024;
					Sector.Tag = 800;
				}
				else if(m_iConversionType == CTYPE_DOOM || m_iConversionType == CTYPE_EDGE)
					Sector.Special = 9;

				AddSector(&Sector);

				f = lpFloor[y * 64 + x];

				int SectorID = 800 + m_iNextSecret++;

				//Old way of doing secret walls with models...
				/*
				Secret.m_ID = SectorID;
				Secret.m_Options = 2023;
				Secret.m_Type = 22000 + f;
				PositionThing (&Secret, x, y);
				AddThing(&Secret);
				*/

				bDoor = FALSE;

				lstrcpy(Line.m_sRight.szUpper,"-");
				lstrcpy(Line.m_sRight.szMiddle,"-");
				lstrcpy(Line.m_sRight.szLower,"-");

				b = lpFloor[y * 64 + (x - 1)];	// Left
				if(lpObject[y * 64 + (x - 1)] == 98) bDoor = TRUE;
				if(b >= 88)
				{
					if(b <= 105)
						bDoor = TRUE;
					b = f;
					fSides |= 1;
					iDir = 0;
				}
				Line.Reset();
				PositionLine(&Line,x,y,LINE_LEFT);
				AddSecretLine(&Line,s,b, SectorID, bDoor);

				bDoor = FALSE;

				b = lpFloor[y * 64 + (x + 1)];	// Right
				if(lpObject[y * 64 + (x + 1)] == 98) bDoor = TRUE;
				if(b >= 88)
				{
					if(b <= 105)
						bDoor = TRUE;
					b = f;
					fSides |= 2;
					iDir = 0;
				}
				Line.Reset();
				PositionLine(&Line,x,y,LINE_RIGHT);
				AddSecretLine(&Line,s,b, SectorID, bDoor);

				bDoor = FALSE;

				b = lpFloor[(y - 1) * 64 + x];	// Above
				if(lpObject[(y - 1) * 64 + x] == 98) bDoor = TRUE;
				if(b >= 88)
				{
					if(b <= 105)
						bDoor = TRUE;
					b = f;
					fSides |= 4;
					iDir = 1;
				}
				Line.Reset();
				PositionLine(&Line,x,y,LINE_ABOVE);
				AddSecretLine(&Line,s,b, SectorID, bDoor);

				bDoor = FALSE;

				b = lpFloor[(y + 1) * 64 + x];	// Below
				if(lpObject[(y + 1) * 64 + x] == 98) bDoor = TRUE;
				if(b >= 88)
				{
					if(b <= 105)
						bDoor = TRUE;
					b = f;
					fSides |= 8;
					iDir = 1;
				}
				Line.Reset();
				PositionLine(&Line,x,y,LINE_BELOW);
				AddSecretLine(&Line,s,b, SectorID, bDoor);

				// TJL
				
				if(m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
				{
					LPPOLY_QUEUE lpNew = new POLY_QUEUE;
					Line.GetMapLine2(&lpNew->Door1);

					lpNew->iType = SECRET;
					lpNew->bDir = iDir;
					lpNew->Next = NULL;
					lpNew->iTexture = f;

					if(m_lpQueue)
					{
						LPPOLY_QUEUE lpList = m_lpQueue;
						while(lpList->Next)
							lpList = lpList->Next;
						lpList->Next = lpNew;
					}
					else
						m_lpQueue = lpNew;
				}
				//TJL */	
			}
		}
		lpRow += 64;
	}
}

VOID CMapConvert::BuildDoorBS (INT x, INT y, INT iType, INT bDir, INT bSilence, INT iWallType)
{
	CMapLine Ent1, Ent2, Track1, Track2, Door1, Door2, Door3, Door4, DoorSplit;
	CMapLine Split1, Split2, Split3, Split4;
	CMapLine Socket1, Socket2, Socket3, Socket4;	// TJL - Added
	CMapLine* lpList[17];		// TJL - Increased for Socket1 and Socket2
	SECTOR Sector, Sector2, Sector3;
	INT i;
	lpList[0] = &Ent1;
	lpList[1] = &Ent2;
	lpList[2] = &Door1;
	lpList[3] = &Door3;
	lpList[4] = &DoorSplit;
	lpList[5] = &Door2;
	lpList[6] = &Door4;
	lpList[7] = &Track1;
	lpList[8] = &Track2;
	lpList[9] = &Split1;
	lpList[10] = &Split2;
	lpList[11] = &Split3;
	lpList[12] = &Split4;
	lpList[13] = &Socket1;
	lpList[14] = &Socket2;
	lpList[15] = &Socket3;
	lpList[16] = &Socket4;

	for(i = 0; i < 16; i++)
	{
		lpList[i]->Reset();
		lstrcpy(lpList[i]->m_sRight.szUpper,"-");
		lstrcpy(lpList[i]->m_sRight.szMiddle,"-");
		lstrcpy(lpList[i]->m_sRight.szLower,"-");
		CopyMemory(&lpList[i]->m_sLeft,&lpList[i]->m_sRight,sizeof(SIDEDEF));
	}
	if(bDir == 0)
	{
		PositionLine(&Ent1,x,y,LINE_LEFT);
		PositionLine(&Ent2,x,y,LINE_RIGHT);
		PositionLine(&Track1,x,y,LINE_ABOVE);
		PositionLine(&Track2,x,y,LINE_BELOW);
		Door1 = Ent1;
		Door2 = Ent2;
		Door1.m_vFrom.x += 28;
		Door1.m_vTo.x += 28;
		Door2.m_vFrom.x -= 28;
		Door2.m_vTo.x -= 28;
		Door3 = Door1;
		Door4 = Door2;
		Split1 = Track1;
		Split2 = Track1;
		Split3 = Track2;
		Split4 = Track2;

		Door1.m_vTo.y -= 32;
		Door2.m_vTo.y += 32;

		Socket1 = Door1;
		Socket2 = Door2;
		Socket1.m_vFrom.y -= 32;
		Socket1.m_vTo.y -= 32;
		Socket2.m_vFrom.y += 32;
		Socket2.m_vTo.y += 32;

		Door3.m_vFrom.y += 32;
		Door4.m_vFrom.y -= 32;

		Socket3 = Door3;
		Socket4 = Door4;
		Socket3.m_vFrom.y += 32;
		Socket3.m_vTo.y += 32;
		Socket4.m_vFrom.y -= 32;
		Socket4.m_vTo.y -= 32;

		Split1.m_vTo.x = Split1.m_vFrom.x + 28;
		Split2.m_vFrom.x = Split2.m_vTo.x - 28;
		Split3.m_vTo.x = Split3.m_vFrom.x - 28;
		Split4.m_vFrom.x = Split4.m_vTo.x + 28;

		Track1.m_vFrom = Socket1.m_vFrom;
		Track1.m_vTo = Socket4.m_vTo;
		Track2.m_vFrom = Socket3.m_vTo;
		Track2.m_vTo = Socket2.m_vFrom;
		DoorSplit = Track2;
		DoorSplit.m_vFrom.y -= 64;
		DoorSplit.m_vTo.y -= 64;
	}
	else
	{
		PositionLine(&Ent1,x,y,LINE_ABOVE);
		PositionLine(&Ent2,x,y,LINE_BELOW);
		PositionLine(&Track1,x,y,LINE_LEFT);
		PositionLine(&Track2,x,y,LINE_RIGHT);
		Door1 = Ent1;
		Door2 = Ent2;
		Door1.m_vFrom.y -= 28;
		Door1.m_vTo.y -= 28;
		Door2.m_vFrom.y += 28;
		Door2.m_vTo.y += 28;
		Door3 = Door1;
		Door4 = Door2;
		Split1 = Track1;
		Split2 = Track1;
		Split3 = Track2;
		Split4 = Track2;

		Door1.m_vTo.x -= 32;
		Door2.m_vTo.x += 32;

		Socket1 = Door1;
		Socket2 = Door2;
		Socket1.m_vFrom.x -= 32;
		Socket1.m_vTo.x -= 32;
		Socket2.m_vFrom.x += 32;
		Socket2.m_vTo.x += 32;

		Door3.m_vFrom.x += 32;
		Door4.m_vFrom.x -= 32;

		Socket3 = Door3;
		Socket4 = Door4;
		Socket3.m_vFrom.x += 32;
		Socket3.m_vTo.x += 32;
		Socket4.m_vFrom.x -= 32;
		Socket4.m_vTo.x -= 32;

		Split1.m_vTo.y = Split1.m_vFrom.y + 28;
		Split2.m_vFrom.y = Split2.m_vTo.y - 28;
		Split3.m_vTo.y = Split3.m_vFrom.y - 28;
		Split4.m_vFrom.y = Split4.m_vTo.y + 28;

		Track1.m_vFrom = Socket1.m_vFrom;
		Track1.m_vTo = Socket4.m_vTo;
		Track2.m_vFrom = Socket3.m_vTo;
		Track2.m_vTo = Socket2.m_vFrom;
		DoorSplit = Track2;
		DoorSplit.m_vFrom.x -= 64;
		DoorSplit.m_vTo.x -= 64;
	}

	FlipLinedef(&Door1);
	FlipLinedef(&Door2);
	FlipLinedef(&Door3);
	FlipLinedef(&Door4);

	if(m_bUseUpperPanels)
	{
		lstrcpy(Ent1.m_sLeft.szUpper,"NZPANEL");
		lstrcpy(Ent2.m_sLeft.szUpper,"NZPANEL");
	}

	// Door tracks
	//lstrcpy(Track1.m_sRight.szMiddle,"DOORTRAK");
	//lstrcpy(Track2.m_sRight.szMiddle,"DOORTRAK");
	Track1.m_Flags = 17;	// Impassible & Lower unpegged
	Track2.m_Flags = 17;	// Impassible & Lower unpegged
	Socket1.m_Flags = 17;	// Impassible & Lower unpegged
	Socket2.m_Flags = 17;	// Impassible & Lower unpegged
	Socket3.m_Flags = 17;	// Impassible & Lower unpegged
	Socket4.m_Flags = 17;	// Impassible & Lower unpegged

	for(i = 7; i <= 16; i++)
	{
		switch(iWallType)
		{
		case 90:	//Grey/Blue N/S
		case 94:	//Elevator N/S
			lstrcpy(lpList[i]->m_sRight.szMiddle,"BSDFD");
			break;
		case 91:	//Grey/Blue E/W
		case 95:	//Elevator E/W
			lstrcpy(lpList[i]->m_sRight.szMiddle,"BSDFD2");
			break;
		case 92:	//Blue/Blue N/S
		case 96:	//Blue/Red N/S
		case 101:	//1-Way \/
		case 99:	//1-Way ^
			lstrcpy(lpList[i]->m_sRight.szMiddle,"BSDF");
			break;
		case 93:	//Blue/Blue E/W
		case 97:	//Blue/Red E/W
		case 98:	//1-Way <
		case 100:	//1-Way >
			lstrcpy(lpList[i]->m_sRight.szMiddle,"BSDF2");
			break;
		case 102:	//Brown N/S
			lstrcpy(lpList[i]->m_sRight.szMiddle,"BSDFB");
			break;
		case 103:	//Brown E/W
			lstrcpy(lpList[i]->m_sRight.szMiddle,"BSDFB2");
			break;
		case 88:	//White Door N/S
			lstrcpy(lpList[i]->m_sRight.szMiddle,"BSDFA");
			break;
		case 89:	//White Door E/W
			lstrcpy(lpList[i]->m_sRight.szMiddle,"BSDFA2");
			break;
		case 104:	//Grey/Grey N/S
			lstrcpy(lpList[i]->m_sRight.szMiddle,"BSDFC");
			break;
		case 105:	//Grey/Grey E/W
			lstrcpy(lpList[i]->m_sRight.szMiddle,"BSDFC2");
			break;
		default:
			lstrcpy(lpList[i]->m_sRight.szMiddle,"BSDF");
			break;
		}
		lpList[i]->m_Flags = 1;
		lpList[i]->m_sRight.yOffset = 0;
		lpList[i]->m_sRight.Sector = m_iNextSector;
	}

	Split2.m_sRight.xOffset = -28;
	Split4.m_sRight.xOffset = -28;
	Track1.m_sRight.xOffset = 28;
	Track2.m_sRight.xOffset = 28;

	// Sector
	FillSector(&Sector);
	Sector.Ceiling = 64;
	AddSector(&Sector);

	FlipLinedef(&Ent1);
	FlipLinedef(&Ent2);
	FlipLinedef(&Track1);

	// Sector Assignment
	Ent1.m_sLeft.Sector = m_iNextSector;
	Ent2.m_sLeft.Sector = m_iNextSector;
	Door1.m_sRight.Sector = m_iNextSector;
	Door2.m_sRight.Sector = m_iNextSector;
	Door3.m_sRight.Sector = m_iNextSector;
	Door4.m_sRight.Sector = m_iNextSector;

	m_iNextSector++;
	Socket1.m_sRight.Sector = m_iNextSector;
	Socket4.m_sRight.Sector = m_iNextSector;
	Track1.m_sRight.Sector = m_iNextSector;
	Door1.m_sLeft.Sector = m_iNextSector;
	Door4.m_sLeft.Sector = m_iNextSector;
	DoorSplit.m_sRight.Sector = m_iNextSector;

	FillSector(&Sector2);
	Sector2.Ceiling = 64;
	AddSector(&Sector2);

	m_iNextSector++;
	Socket2.m_sRight.Sector = m_iNextSector;
	Socket3.m_sRight.Sector = m_iNextSector;
	Track2.m_sRight.Sector = m_iNextSector;
	Door3.m_sLeft.Sector = m_iNextSector;
	Door2.m_sLeft.Sector = m_iNextSector;
	DoorSplit.m_sLeft.Sector = m_iNextSector;

	FillSector(&Sector3);
	Sector3.Ceiling = 64;
	AddSector(&Sector3);

	m_iNextSector++;

	// More line stuff
	if (bSilence)		//TJL - Only silence between sector with different floor types
		Ent1.m_Flags = 68;
	else
		Ent1.m_Flags = 4;

	Ent2.m_Flags = 68;
	Door1.m_Flags = 4;
	Door2.m_Flags = 4;
	Door3.m_Flags = 4;
	Door4.m_Flags = 4;
	DoorSplit.m_Flags = 4;

	Door1.m_Special = 226;
	Door1.m_Flags = 1540;
	Door1.m_Args[0] = 1;
	Door1.m_Args[3] = bDir + 1;

	if (iType == DOOR_ONEWAYN || iType == DOOR_ONEWAYE)
		Door1.m_Args[4] = 99;
	else
		Door1.m_Args[4] = iType - 1;

	Door2.m_Special = 226;
	Door2.m_Flags = 1540;
	Door2.m_Args[0] = 1;
	Door2.m_Args[3] = bDir + 1;

	if (iType == DOOR_ONEWAYS || iType == DOOR_ONEWAYW)
		Door2.m_Args[4] = 99;
	else
		Door2.m_Args[4] = iType - 1;

	LPPOLY_QUEUE lpNew = new POLY_QUEUE;
	if(lpNew)
	{
		Door1.GetMapLine2(&lpNew->Door1);
		Door2.GetMapLine2(&lpNew->Door2);

		lpNew->iType = iWallType * 10 + iType;
		lpNew->bDir = bDir;
		lpNew->iPoly = m_iNextPoly++;
		lpNew->Next = NULL;

		if(m_lpQueue)
		{
			LPPOLY_QUEUE lpList = m_lpQueue;
			while(lpList->Next)
				lpList = lpList->Next;
			lpList->Next = lpNew;
		}
		else
			m_lpQueue = lpNew;

		Door1.m_Args[2] = lpNew->iPoly;
		Door2.m_Args[2] = lpNew->iPoly;
		Door3.m_Args[2] = lpNew->iPoly;
		Door4.m_Args[2] = lpNew->iPoly;
	}

	Door3.m_Special = 226;
	Door3.m_Flags = 1540;
	Door3.m_Args[0] = 1;
	Door3.m_Args[3] = bDir + 1;
	if (iType == DOOR_ONEWAYN || iType == DOOR_ONEWAYE)
		Door3.m_Args[4] = 99;
	else
		Door3.m_Args[4] = iType - 1;

	Door3.m_Args[4] = iType - 1;
	Door4.m_Special = 226;
	Door4.m_Flags = 1540;
	Door4.m_Args[0] = 1;
	Door4.m_Args[3] = bDir + 1;
	if (iType == DOOR_ONEWAYS || iType == DOOR_ONEWAYW)
		Door4.m_Args[4] = 99;
	else
		Door4.m_Args[4] = iType - 1;

	LPPOLY_QUEUE lpNew2 = new POLY_QUEUE;
	if(lpNew2)
	{
		Door3.GetMapLine2(&lpNew2->Door1);
		Door4.GetMapLine2(&lpNew2->Door2);

		lpNew2->iType = iWallType * 10 + iType;
		lpNew2->bDir = bDir;
		lpNew2->iPoly = m_iNextPoly++;
		lpNew2->Next = NULL;

		if(m_lpQueue)
		{
			LPPOLY_QUEUE lpList2 = m_lpQueue;
			while(lpList2->Next)
				lpList2 = lpList2->Next;
			lpList2->Next = lpNew2;
		}
		else
			m_lpQueue = lpNew2;
	}

	Ent1.m_Special = 226;
	Ent2.m_Special = 226;
	Ent1.m_Flags |= (512 | 8192);
	Ent2.m_Flags |= (512 | 8192);
	//Ent1.m_Flags |= (2 << 10);
	//Ent2.m_Flags |= (2 << 10);
	Ent1.m_Args[0] = 1;
	Ent2.m_Args[0] = 1;
	Ent1.m_Args[2] = lpNew->iPoly;
	Ent2.m_Args[2] = lpNew->iPoly;
	Ent1.m_Args[3] = bDir + 1;
	Ent2.m_Args[3] = bDir + 1;
	Ent1.m_Args[4] = 98;
	Ent2.m_Args[4] = 98;

	for(i = 0; i <= 16; i++)
		AddLine(lpList[i]);
}

VOID CMapConvert::BuildUsableWalls (USHORT* lpFloor, USHORT* lpObject)
{
	INT x, y, iObject = 0, parameter[4] = {0, 0, 0, 0}, healthcount = 0; //switchedx, switchedy;
	USHORT *lpObj = lpObject, *lpRow = lpFloor, f, o;
	CMapLine Line;
	for(y = 0; y < 64; y++)
	{
		for(x = 0; x < 64; x++)
		{
			f = lpRow[x];
			o = lpObj[x];

			if(m_iConversionType == CTYPE_BSTC && 
				(f == 45 ||		//Barrier Switch Up
				f == 32 ||		//Teleporter
				f == 15 ||		//Food Unit 1
				f == 18 ||		//Food Unit 2
				f == 57 ||		//Barrier Switch Up
				f == 157 ||		//End Mission
				f == 158))		//Autodoor
			{
				switch(f)
				{
				case 45:
				case 57:
					switch(o)
					{
						case 63743:	//switch on same level
							iObject = lpObj[x+1];
							/*
							switchedx = INT(iObject/256);
							switchedy = INT(iObject%256);
							//check to make sure it's the right tag
							iObject = CheckThingNeighbor(lpObject, lpObj[switchedx + (64*(switchedy - y))], switchedx, switchedy, 0);
							*/
							parameter[0] = 0;
						break;
						case 63488: //switch on diffferent level...  0 through 11
						case 63489:
						case 63490:
						case 63491:
						case 63492:
						case 63493:
						case 63494:
						case 63495:
						case 63496:
						case 63497:
						case 63498:
							//Trust that it's set up 'right'...
							iObject = lpObj[x+1];
							parameter[0] = o - 63488;
						break;
					}
				break;
				case 32:
					switch(o)
					{
						case 62464: //Teleporter to Map 0
							iObject = 1;
							parameter[0] = 12;
							break;
					}
					break;
				case 15:	//Food Unit 1
				case 18: 	//Food Unit 2
					if(o > 64500 && o < 64700)
					{
						iObject = 2;
						healthcount++;
						parameter[0] = o - 64512;
						parameter[1] = healthcount;
					}
					else
					{
						iObject = 3;
						parameter[0] = 255;
					}
					break;
				case 157:	//End Mission
					break;
				case 158:	//Autodoor
					break;
				}
				BuildUsable(x,y,iObject,f,parameter);
			}
		}
		lpRow += 64;
		lpObj += 64;
	}
}

VOID CMapConvert::BuildUsable (INT x, INT y, INT iObject, INT iWallType, INT parameter[4])
{
	CMapLine Up, Down, Left, Right;
	CMapLine* lpList[4];
	INT i, dir;
	lpList[0] = &Up;
	lpList[1] = &Down;
	lpList[2] = &Left;
	lpList[3] = &Right;

	PositionLine(&Up,x,y,LINE_ABOVE);
	PositionLine(&Down,x,y,LINE_BELOW);
	PositionLine(&Left,x,y,LINE_LEFT);
	PositionLine(&Right,x,y,LINE_RIGHT);

	for(i = 0; i < 4; i++)
	{
		CMapLine* lpFind = FindLine(&lpList[i]->m_vTo,&lpList[i]->m_vFrom);
		if(lpFind)
		{
			if (lpFind->m_vFrom.y == lpFind->m_vTo.y)
				dir = 1;
			else
				dir = 0;

			lpFind->m_Flags |= 1536;
			lpFind->m_Special = 226;

			switch (iObject)
			{
			case 1:	//Teleporter
				lpFind->m_Args[0] = 41;
				lpFind->m_Args[1] = 0;
				lpFind->m_Args[2] = parameter[0];
				break;
			case 2: //Health (Working)
				lpFind->m_Args[3] = parameter[1];
			case 3: //Health (Not Working)
				lpFind->m_Args[0] = 42;
				lpFind->m_Args[1] = 0;
				lpFind->m_Args[2] = parameter[0];
				break;
			default: //Barrier Switch
				if (parameter[0] == 0)
				{
					lpFind->m_Special = 121;
					lpFind->m_Args[0] = INT (iObject%256);
					lpFind->m_Args[1] = 0;
					lpFind->m_Args[2] = 0;
					lpFind->m_Args[3] = 0;
					lpFind->m_Args[4] = INT (iObject/256);
					if (lpFind->m_vFrom.y == lpFind->m_vTo.y)
						lstrcpy(lpFind->m_sRight.szMiddle,"NZSW");
					else
						lstrcpy(lpFind->m_sRight.szMiddle,"NZSW2");
				}
				else
				{
					lpFind->m_Flags ^= 512;
					lpFind->m_Args[0] = 40;
					lpFind->m_Args[1] = parameter[0];
					lpFind->m_Args[2] = INT (iObject/256);
					lpFind->m_Args[3] = INT (iObject%256);
				}
				break;
			}
			lstrcpy(lpFind->m_sLeft.szLower,"-");
			lstrcpy(lpFind->m_sLeft.szUpper,"-");
		}
	}
}

VOID CMapConvert::BuildDoor (INT x, INT y, INT iType, INT bDir, INT bSilence)
{
	CMapLine Ent1, Ent2, Track1, Track2, Door1, Door2, Door3, Door4;
	CMapLine Split1, Split2, Split3, Split4, Split5;
	CMapLine Socket1, Socket2;	// TJL - Added
	CMapLine* lpList[12];		// TJL - Increased for Socket1 and Socket2
	SECTOR Sector;
	INT i;
	lpList[0] = &Ent1;
	lpList[1] = &Ent2;
	lpList[2] = &Track1;
	lpList[3] = &Track2;
	lpList[4] = &Door1;
	lpList[5] = &Door2;
	lpList[6] = &Split1;
	lpList[7] = &Split2;
	lpList[8] = &Split3;
	lpList[9] = &Split4;
	// TJL - Added
	lpList[10] = &Socket1;
	lpList[11] = &Socket2;
	for(i = 0; i < 12; i++)
	{
		lpList[i]->Reset();
		lstrcpy(lpList[i]->m_sRight.szUpper,"-");
		lstrcpy(lpList[i]->m_sRight.szMiddle,"-");
		lstrcpy(lpList[i]->m_sRight.szLower,"-");
		CopyMemory(&lpList[i]->m_sLeft,&lpList[i]->m_sRight,sizeof(SIDEDEF));
	}

	if(bDir == 0)
	{
		PositionLine(&Ent1,x,y,LINE_LEFT);
		PositionLine(&Ent2,x,y,LINE_RIGHT);
		PositionLine(&Track1,x,y,LINE_ABOVE);
		PositionLine(&Track2,x,y,LINE_BELOW);
		Door1 = Ent1;
		Door2 = Ent2;
		Door1.m_vFrom.x += 28;
		Door1.m_vTo.x += 28;
		Door2.m_vFrom.x -= 28;
		Door2.m_vTo.x -= 28;
		Split1 = Track1;
		Split2 = Track1;
		Split3 = Track2;
		Split4 = Track2;

		// TJL
		Socket1 = Door1;
		Socket2 = Door2;
		Socket1.m_vFrom.y -= 63;
		Socket1.m_vTo.y -= 64;
		Socket2.m_vFrom.y -= 64;
		Socket2.m_vTo.y -= 63;

		Split1.m_vTo.x = Split1.m_vFrom.x + 28;
		Split2.m_vFrom.x = Split2.m_vTo.x - 28;
		Split3.m_vTo.x = Split3.m_vFrom.x - 28;
		Split4.m_vFrom.x = Split4.m_vTo.x + 28;
		Track1.m_vFrom.x = Split1.m_vTo.x;
		Track1.m_vTo.x = Split2.m_vFrom.x;
		Track2.m_vFrom.x = Split3.m_vTo.x;
		Track2.m_vTo.x = Split4.m_vFrom.x;

		if(m_bBuildDoorSockets)
		{
			// TJL
			Track2.m_vFrom.y -= 63;
			Track2.m_vTo.y -= 63;
		}
	}
	else
	{
		PositionLine(&Ent1,x,y,LINE_ABOVE);
		PositionLine(&Ent2,x,y,LINE_BELOW);
		PositionLine(&Track1,x,y,LINE_LEFT);
		PositionLine(&Track2,x,y,LINE_RIGHT);
		Door1 = Ent1;
		Door2 = Ent2;
		Door1.m_vFrom.y -= 28;
		Door1.m_vTo.y -= 28;
		Door2.m_vFrom.y += 28;
		Door2.m_vTo.y += 28;
		Split1 = Track1;
		Split2 = Track1;
		Split3 = Track2;
		Split4 = Track2;

		// TJL
		Socket1 = Door1;
		Socket2 = Door2;
		Socket1.m_vFrom.x -= 63;
		Socket1.m_vTo.x -= 64;
		Socket2.m_vFrom.x -= 64;
		Socket2.m_vTo.x -= 63;

		Split1.m_vTo.y = Split1.m_vFrom.y + 28;
		Split2.m_vFrom.y = Split2.m_vTo.y - 28;
		Split3.m_vTo.y = Split3.m_vFrom.y - 28;
		Split4.m_vFrom.y = Split4.m_vTo.y + 28;
		Track1.m_vFrom.y = Split1.m_vTo.y;
		Track1.m_vTo.y = Split2.m_vFrom.y;
		Track2.m_vFrom.y = Split3.m_vTo.y;
		Track2.m_vTo.y = Split4.m_vFrom.y;

		if(m_bBuildDoorSockets)
		{
			// TJL
			Track1.m_vFrom.x -= 63;
			Track1.m_vTo.x -= 63;
		}
	}
	FlipLinedef(&Door1);
	FlipLinedef(&Door2);
	if(m_bUseUpperPanels)
	{
		lstrcpy(Ent1.m_sLeft.szUpper,"NZPANEL");
		lstrcpy(Ent2.m_sLeft.szUpper,"NZPANEL");
	}
	// Door tracks
	lstrcpy(Track1.m_sRight.szMiddle,"DOORTRAK");
	lstrcpy(Track2.m_sRight.szMiddle,"DOORTRAK");
	Track1.m_Flags = 17;	// Impassible & Lower unpegged
	Track2.m_Flags = 17;	// Impassible & Lower unpegged
	// TJL - Added
	Socket1.m_Flags = 17;	// Impassible & Lower unpegged
	Socket2.m_Flags = 17;	// Impassible & Lower unpegged
	// Door frames
	for(i = 6; i < 12; i++)
	{
		if (lpList[i]->m_vFrom.y == lpList[i]->m_vTo.y)
			lstrcpy(lpList[i]->m_sRight.szMiddle,"NZDFRAME");
		else
			lstrcpy(lpList[i]->m_sRight.szMiddle,"NZDFRAME2");
		lpList[i]->m_Flags = 1;
		lpList[i]->m_sRight.yOffset = 0;
		lpList[i]->m_sRight.Sector = m_iNextSector;
	}
	Split2.m_sRight.xOffset = -28;
	Split4.m_sRight.xOffset = -28;
	// Sector
	FillSector(&Sector);
	Sector.Ceiling = 64;
	AddSector(&Sector);
	if(m_iConversionType == CTYPE_DOOM || m_iConversionType == CTYPE_EDGE)
	{
		Sector.Ceiling = 0;
		SetFrontDoor(iType,Door1.m_sRight.szUpper, bDir);
		SetBackDoor(iType,Door2.m_sRight.szUpper, bDir);
		switch(iType)
		{
		case DOOR_SILVER:
			Door1.m_Types = 26;
			Door2.m_Types = 26;
			break;
		case DOOR_GOLD:
			Door1.m_Types = 27;
			Door2.m_Types = 27;
			break;
		default:
			Door1.m_Types = 1;
			Door2.m_Types = 1;
			break;
		}
	}
	AddSector(&Sector);

	// Sector Assignment
	Ent1.m_sRight.Sector = m_iNextSector;
	Ent2.m_sRight.Sector = m_iNextSector;
	Ent1.m_sLeft.Sector = -1;
	Ent2.m_sLeft.Sector = -1;
	Door1.m_sRight.Sector = m_iNextSector;
	Door2.m_sRight.Sector = m_iNextSector;

	m_iNextSector++;
	Track1.m_sRight.Sector = m_iNextSector;
	Track2.m_sRight.Sector = m_iNextSector;
	Door1.m_sLeft.Sector = m_iNextSector;
	Door2.m_sLeft.Sector = m_iNextSector;

	// TJL
	Socket1.m_sRight.Sector = m_iNextSector;
	Socket2.m_sRight.Sector = m_iNextSector;

	m_iNextSector++;

	// More line stuff
	if (bSilence)		//TJL - Only silence between sector with different floor types
		Ent1.m_Flags = 68;
	else
		Ent1.m_Flags = 4;

	Ent2.m_Flags = 68;
	Door1.m_Flags = 4;
	Door2.m_Flags = 4;
	FlipLinedef(&Ent1);
	FlipLinedef(&Ent2);

	if(m_iConversionType == CTYPE_ZDOOM || m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
	{
		LPPOLY_QUEUE lpNew = new POLY_QUEUE;
		if(lpNew)
		{
			Door1.GetMapLine2(&lpNew->Door1);
			Door2.GetMapLine2(&lpNew->Door2);

			lpNew->iType = iType;
			lpNew->bDir = bDir;
			lpNew->iPoly = m_iNextPoly++;
			lpNew->Next = NULL;

			if(m_lpQueue)
			{
				LPPOLY_QUEUE lpList = m_lpQueue;
				while(lpList->Next)
					lpList = lpList->Next;
				lpList->Next = lpNew;
			}
			else
				m_lpQueue = lpNew;

			if(m_bUseAutoDoors)
			{
				Ent1.m_Special = 226;
				Ent2.m_Special = 226;
				Ent1.m_Flags |= (512 | 8192);
				Ent2.m_Flags |= (512 | 8192);
				//Ent1.m_Flags |= (2 << 10);
				//Ent2.m_Flags |= (2 << 10);
				Ent1.m_Args[0] = 1;
				Ent2.m_Args[0] = 1;
				Ent1.m_Args[2] = lpNew->iPoly;
				Ent2.m_Args[2] = lpNew->iPoly;
				Ent1.m_Args[3] = bDir + 1;
				Ent2.m_Args[3] = bDir + 1;
				Ent1.m_Args[4] = 5;
				Ent2.m_Args[4] = 5;
			}

			Door1.m_Special = 226;
			Door1.m_Flags = 1540;
			Door1.m_Args[0] = 1;
			Door1.m_Args[2] = lpNew->iPoly;
			Door1.m_Args[3] = bDir + 1;
			Door1.m_Args[4] = iType - 1;
			Door2.m_Special = 226;
			Door2.m_Flags = 1540;
			Door2.m_Args[0] = 1;
			Door2.m_Args[2] = lpNew->iPoly;
			Door2.m_Args[3] = bDir + 1;
			Door2.m_Args[4] = iType - 1;
		}
	}

	if(m_bBuildDoorSockets)
	{
		for(i = 0; i < 12; i++)
			//TJL - Added check for doors next to doors...
			// If the entryway lines already exist, make them double-sided/passable
			if (i == 0 || i == 1)
			{
				CMapLine* lpFind = FindLine(&lpList[i]->m_vTo,&lpList[i]->m_vFrom);
				if (lpFind)
				{
					AddSidedLine(lpList[i], lpList[i]->m_sRight.Sector, '-');
				}
			}
			else
			{
				AddLine(lpList[i]);
			}
	}
	else
	{
		for(i = 0; i < 10; i++)
			AddLine(lpList[i]);
	}
}

VOID CMapConvert::BuildPolyDoor (CMapLine* lpDoor1, CMapLine* lpDoor2, INT iType, INT bDir, INT iTexture, INT iPoly, INT Side, INT iWallType)
{
	if(m_iConversionType == CTYPE_ZDOOM || m_iConversionType == CTYPE_ZDOOMTC)
	{
		CMapLine Line[5];
		CMapThing Anchor, Start;
		INT i, sParent;
		BOOL bCreateContainer = FALSE;

		if(bDir == 0 && m_iHorzStrip == 0)
		{	// east/west
			Line[0].m_vFrom.x = -2048;
			Line[0].m_vFrom.y = 2240;
			Line[0].m_vTo.x = 2048;
			Line[0].m_vTo.y = 2240;
			CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
			Line[1].m_vTo.x = 2048;
			Line[1].m_vTo.y = 2112;
			CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
			Line[2].m_vTo.x = -2048;
			Line[2].m_vTo.y = 2112;
			m_iHorzStrip = m_iNextSector;
			bCreateContainer = TRUE;
		}
		else if(bDir == 1 && m_iVertStrip == 0)
		{	// north/south
			Line[0].m_vFrom.x = -2240;
			Line[0].m_vFrom.y = 2048;
			Line[0].m_vTo.x = -2112;
			Line[0].m_vTo.y = 2048;
			CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
			Line[1].m_vTo.x = -2112;
			Line[1].m_vTo.y = -2048;
			CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
			Line[2].m_vTo.x = -2240;
			Line[2].m_vTo.y = -2048;
			m_iVertStrip = m_iNextSector;
			bCreateContainer = TRUE;
		}

		if(bCreateContainer)
		{
			SECTOR Sector;
			FillSector(&Sector);
			Sector.Ceiling = 64;
			CopyMemory(&Line[3].m_vFrom,&Line[2].m_vTo,sizeof(VERTEX));
			CopyMemory(&Line[3].m_vTo,&Line[0].m_vFrom,sizeof(VERTEX));
			for(i = 0; i < 4; i++)
			{
				lstrcpy(Line[i].m_sRight.szLower,"-");
				//TJL - Remove all references to Doom textures if TC
				if(m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
					lstrcpy(Line[i].m_sRight.szMiddle,"FLOOR");
				else
					lstrcpy(Line[i].m_sRight.szMiddle,"STONE2");
				lstrcpy(Line[i].m_sRight.szUpper,"-");
				Line[i].m_Flags = 1;
				Line[i].m_sRight.Sector = m_iNextSector;
				Line[i].m_sLeft.Sector = -1;
				AddLine(&Line[i]);
			}
			m_iNextSector++;
			AddSector(&Sector);
		}

		if(iType == SECRET)
		{
			/*
			if(bDir == 0)
			{	// east/west
				Line[0].m_vFrom.x = m_xNextDoor;
				Line[0].m_vFrom.y = 2208;
				Line[0].m_vTo.x = m_xNextDoor;
				Line[0].m_vTo.y = 2144;
				CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
				Line[1].m_vTo.x = m_xNextDoor + 64;
				Line[1].m_vTo.y = 2144;
				CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
				Line[2].m_vTo.x = m_xNextDoor + 64;
				Line[2].m_vTo.y = 2208;
				m_xNextDoor += 96;
				Anchor.m_x = Line[1].m_vFrom.x + 32;
				Anchor.m_y = Line[1].m_vFrom.y + 8;
				Start.m_x = lpDoor1->m_vTo.x + 32;
				Start.m_y = lpDoor1->m_vTo.y + 8;
				sParent = m_iHorzStrip;
			}
			else if(bDir == 1)
			{	// north/south
			*/
				Line[0].m_vFrom.x = -2144;
				Line[0].m_vFrom.y = m_yNextDoor;
				Line[0].m_vTo.x = -2208;
				Line[0].m_vTo.y = m_yNextDoor;
				CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
				Line[1].m_vTo.x = -2208;
				Line[1].m_vTo.y = m_yNextDoor - 64;
				CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
				Line[2].m_vTo.x = -2144;
				Line[2].m_vTo.y = m_yNextDoor - 64;
				m_yNextDoor -= 96;
				Anchor.m_x = Line[1].m_vFrom.x + 8;
				Anchor.m_y = Line[1].m_vFrom.y - 32;
				Start.m_x = lpDoor1->m_vTo.x + 8;
				Start.m_y = lpDoor1->m_vTo.y + 32;
				sParent = m_iVertStrip;
			//}

			CopyMemory(&Line[3].m_vFrom,&Line[2].m_vTo,sizeof(VERTEX));
			CopyMemory(&Line[3].m_vTo,&Line[0].m_vFrom,sizeof(VERTEX));

			Line[1].m_vTo.y = Line[1].m_vTo.y + 1;
			CopyMemory(&Line[4].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
			CopyMemory(&Line[4].m_vTo,&Line[2].m_vFrom,sizeof(VERTEX));

			Line[0].m_Args[3] = 192;
			Line[1].m_Args[3] = 0;
			Line[2].m_Args[3] = 64;
			Line[3].m_Args[3] = 128;

			if(iTexture < 16)
			{
				for(i = 0; i < 5; i++)
				{
					if (Line[i].m_vFrom.y == Line[i].m_vTo.y)
						wsprintf(Line[i].m_sRight.szMiddle,"NZ0%X",iTexture);
					else
						wsprintf(Line[i].m_sRight.szMiddle,"NZ0%X2",iTexture);
				}
			}
			else
			{
				for(i = 0; i < 5; i++)
				{
					if (Line[i].m_vFrom.y == Line[i].m_vTo.y)
						wsprintf(Line[i].m_sRight.szMiddle,"NZ%X",iTexture);
					else
						wsprintf(Line[i].m_sRight.szMiddle,"NZ%X2",iTexture);
				}
			}

			for(i = 0; i < 4; i++)
			{
				Line[i].m_Flags = 9729;
				Line[i].m_Special = 226;
				Line[i].m_Args[0] = 2;
				Line[i].m_Args[2] = m_iNextPoly;
			}

			Line[4].m_Flags = 1;
			Line[4].m_Special = 1;
			Line[4].m_Args[0] = m_iNextPoly;

			for(i = 0; i < 5; i++)
			{
				lstrcpy(Line[i].m_sRight.szLower,"-");
				lstrcpy(Line[i].m_sRight.szUpper,"-");
				Line[i].m_sRight.Sector = sParent;
				Line[i].m_sLeft.Sector = -1;
				AddLine(&Line[i]);
			}

			Anchor.m_Angle = m_iNextPoly;
			Anchor.m_Options = 2023;
			Anchor.m_Type = 9300;
			AddThing(&Anchor);
			Start.m_Angle = m_iNextPoly;
			Start.m_Options = 2023;
			Start.m_Type = 9301;
			AddThing(&Start);
			m_iNextPoly++;
		}
		else
		{
			if(bDir == 0)
			{	// east/west
				Line[0].m_vFrom.x = m_xNextDoor;
				Line[0].m_vFrom.y = 2208;
				Line[0].m_vTo.x = m_xNextDoor;
				Line[0].m_vTo.y = 2144;
				CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
				Line[1].m_vTo.x = m_xNextDoor + 8;
				Line[1].m_vTo.y = 2144;
				CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
				Line[2].m_vTo.x = m_xNextDoor + 8;
				Line[2].m_vTo.y = 2208;
				m_xNextDoor += 32;
				Anchor.m_x = Line[1].m_vFrom.x + 4;
				Anchor.m_y = Line[1].m_vFrom.y;
				Start.m_x = lpDoor1->m_vTo.x + 4;
				Start.m_y = lpDoor1->m_vTo.y;
				sParent = m_iHorzStrip;
			}
			else if(bDir == 1)
			{	// north/south
				Line[0].m_vFrom.x = -2144;
				Line[0].m_vFrom.y = m_yNextDoor;
				Line[0].m_vTo.x = -2208;
				Line[0].m_vTo.y = m_yNextDoor;
				CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
				Line[1].m_vTo.x = -2208;
				Line[1].m_vTo.y = m_yNextDoor - 8;
				CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
				Line[2].m_vTo.x = -2144;
				Line[2].m_vTo.y = m_yNextDoor - 8;
				m_yNextDoor -= 32;
				Anchor.m_x = Line[1].m_vFrom.x;
				Anchor.m_y = Line[1].m_vFrom.y - 4;
				Start.m_x = lpDoor1->m_vTo.x;
				Start.m_y = lpDoor1->m_vTo.y - 4;
				sParent = m_iVertStrip;
			}
			CopyMemory(&Line[3].m_vFrom,&Line[2].m_vTo,sizeof(VERTEX));
			CopyMemory(&Line[3].m_vTo,&Line[0].m_vFrom,sizeof(VERTEX));
			SetFrontDoor(iType,Line[0].m_sRight.szMiddle, bDir);
			CopyMemory(Line[1].m_sRight.szMiddle,"DOORTRAK",8);
			SetBackDoor(iType,Line[2].m_sRight.szMiddle, bDir);
			CopyMemory(Line[3].m_sRight.szMiddle,"DOORTRAK",8);
			
			Line[0].m_Special = 226;
			Line[0].m_Flags = 1540;
			Line[0].m_Args[0] = 1;
			Line[0].m_Args[2] = iPoly;
			Line[0].m_Args[3] = bDir + 1;
			Line[0].m_Args[4] = iType - 1;
			Line[2].m_Special = 226;
			Line[2].m_Flags = 1540;
			Line[2].m_Args[0] = 1;
			Line[2].m_Args[2] = iPoly;
			Line[2].m_Args[3] = bDir + 1;
			Line[2].m_Args[4] = iType - 1;
			
			Line[3].m_Special = 1;
			Line[3].m_Args[0] = iPoly;
			for(i = 0; i < 4; i++)
			{
				lstrcpy(Line[i].m_sRight.szLower,"-");
				lstrcpy(Line[i].m_sRight.szUpper,"-");
				if((i & 1) == 0)
					Line[i].m_Flags = 9729;
				else
					Line[i].m_Flags = 1;
				Line[i].m_sRight.Sector = sParent;
				Line[i].m_sLeft.Sector = -1;
				AddLine(&Line[i]);
			}
			Anchor.m_Angle = iPoly;
			Anchor.m_Options = 2023;
			Anchor.m_Type = 9300;
			AddThing(&Anchor);
			Start.m_Angle = iPoly;
			Start.m_Options = 2023;
			Start.m_Type = 9301;
			AddThing(&Start);
		}
		/*
		if(bDir == 0)
		{	// east/west
			Line[0].m_vFrom.x = m_xNextDoor;
			Line[0].m_vFrom.y = 2208;
			Line[0].m_vTo.x = m_xNextDoor;
			Line[0].m_vTo.y = 2144;
			CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
			Line[1].m_vTo.x = m_xNextDoor + 8;
			Line[1].m_vTo.y = 2144;
			CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
			Line[2].m_vTo.x = m_xNextDoor + 8;
			Line[2].m_vTo.y = 2208;
			m_xNextDoor += 32;
			Anchor.m_x = Line[1].m_vFrom.x + 4;
			Anchor.m_y = Line[1].m_vFrom.y;
			Start.m_x = lpDoor1->m_vTo.x + 4;
			Start.m_y = lpDoor1->m_vTo.y;
			sParent = m_iHorzStrip;
		}
		else if(bDir == 1)
		{	// north/south
			Line[0].m_vFrom.x = -2144;
			Line[0].m_vFrom.y = m_yNextDoor;
			Line[0].m_vTo.x = -2208;
			Line[0].m_vTo.y = m_yNextDoor;
			CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
			Line[1].m_vTo.x = -2208;
			Line[1].m_vTo.y = m_yNextDoor - 8;
			CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
			Line[2].m_vTo.x = -2144;
			Line[2].m_vTo.y = m_yNextDoor - 8;
			m_yNextDoor -= 32;
			Anchor.m_x = Line[1].m_vFrom.x;
			Anchor.m_y = Line[1].m_vFrom.y - 4;
			Start.m_x = lpDoor1->m_vTo.x;
			Start.m_y = lpDoor1->m_vTo.y - 4;
			sParent = m_iVertStrip;
		}
		CopyMemory(&Line[3].m_vFrom,&Line[2].m_vTo,sizeof(VERTEX));
		CopyMemory(&Line[3].m_vTo,&Line[0].m_vFrom,sizeof(VERTEX));
		 (iType,Line[0].m_sRight.szMiddle);
		CopyMemory(Line[1].m_sRight.szMiddle,"DOORTRAK",8);
		SetBackDoor(iType,Line[2].m_sRight.szMiddle);
		CopyMemory(Line[3].m_sRight.szMiddle,"DOORTRAK",8);
		Line[0].m_Special = 226;
		Line[0].m_Args[0] = 1;
		Line[0].m_Args[2] = iPoly;
		Line[0].m_Args[3] = bDir + 1;
		Line[0].m_Args[4] = iType - 1;
		Line[2].m_Special = 226;
		Line[2].m_Args[0] = 1;
		Line[2].m_Args[2] = iPoly;
		Line[2].m_Args[3] = bDir + 1;
		Line[2].m_Args[4] = iType - 1;
		Line[3].m_Special = 1;
		Line[3].m_Args[0] = iPoly;
		for(i = 0; i < 4; i++)
		{
			lstrcpy(Line[i].m_sRight.szLower,"-");
			lstrcpy(Line[i].m_sRight.szUpper,"-");
			if((i & 1) == 0)
				Line[i].m_Flags = 9217;	// TJL - was 9729
			else
				Line[i].m_Flags = 1;
			Line[i].m_sRight.Sector = sParent;
			Line[i].m_sLeft.Sector = -1;
			AddLine(&Line[i]);
		}
		Anchor.m_Angle = iPoly;
		Anchor.m_Options = 2023;
		Anchor.m_Type = 9300;
		AddThing(&Anchor);
		Start.m_Angle = iPoly;
		Start.m_Options = 2023;
		Start.m_Type = 9301;
		AddThing(&Start);
		*/
	}
	else if(m_iConversionType = CTYPE_BSTC)
	{
		CMapLine Line[4];
		CMapThing Anchor, Start;
		INT i, sParent;
		BOOL bCreateContainer = FALSE;

		if(bDir == 0 && m_iHorzStrip == 0)
		{	// east/west
			Line[0].m_vFrom.x = -2048;
			Line[0].m_vFrom.y = 2240;
			Line[0].m_vTo.x = 2048;
			Line[0].m_vTo.y = 2240;
			CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
			Line[1].m_vTo.x = 2048;
			Line[1].m_vTo.y = 2112;
			CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
			Line[2].m_vTo.x = -2048;
			Line[2].m_vTo.y = 2112;
			m_iHorzStrip = m_iNextSector;
			bCreateContainer = TRUE;
		}
		else if(bDir == 1 && m_iVertStrip == 0)
		{	// north/south
			Line[0].m_vFrom.x = -2240;
			Line[0].m_vFrom.y = 2048;
			Line[0].m_vTo.x = -2112;
			Line[0].m_vTo.y = 2048;
			CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
			Line[1].m_vTo.x = -2112;
			Line[1].m_vTo.y = -2048;
			CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
			Line[2].m_vTo.x = -2240;
			Line[2].m_vTo.y = -2048;
			m_iVertStrip = m_iNextSector;
			bCreateContainer = TRUE;
		}

		if(bCreateContainer)
		{
			SECTOR Sector;
			FillSector(&Sector);
			Sector.Ceiling = 64;
			CopyMemory(&Line[3].m_vFrom,&Line[2].m_vTo,sizeof(VERTEX));
			CopyMemory(&Line[3].m_vTo,&Line[0].m_vFrom,sizeof(VERTEX));
			for(i = 0; i < 4; i++)
			{
				lstrcpy(Line[i].m_sRight.szLower,"-");
				lstrcpy(Line[i].m_sRight.szMiddle,"FLOOR");
				lstrcpy(Line[i].m_sRight.szUpper,"-");
				Line[i].m_Flags = 1;
				Line[i].m_sRight.Sector = m_iNextSector;
				Line[i].m_sLeft.Sector = -1;
				AddLine(&Line[i]);
			}
			m_iNextSector++;
			AddSector(&Sector);
		}

		if(bDir == 0)
		{	// east/west
			if (Side==1)
			{
				Line[0].m_vFrom.x = m_xNextDoor;
				Line[0].m_vFrom.y = 2176;
				Line[0].m_vTo.x = m_xNextDoor;
				Line[0].m_vTo.y = 2144;
				CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
				Line[1].m_vTo.x = m_xNextDoor + 8;
				Line[1].m_vTo.y = 2144;
				CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
				Line[2].m_vTo.x = m_xNextDoor + 8;
				Line[2].m_vTo.y = 2176;
			}
			else
			{
				Line[0].m_vFrom.x = m_xNextDoor;
				Line[0].m_vFrom.y = 2208;
				Line[0].m_vTo.x = m_xNextDoor;
				Line[0].m_vTo.y = 2176;
				CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
				Line[1].m_vTo.x = m_xNextDoor + 8;
				Line[1].m_vTo.y = 2176;
				CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
				Line[2].m_vTo.x = m_xNextDoor + 8;
				Line[2].m_vTo.y = 2208;
			}
			m_xNextDoor += 32;

			Anchor.m_x = Line[1].m_vFrom.x + 4;
			if (Side==1)
			{
				Anchor.m_y = Line[2].m_vFrom.y;
				Start.m_y = lpDoor1->m_vTo.y;
			}
			else
			{
				Anchor.m_y = Line[2].m_vTo.y;
				Start.m_y = lpDoor1->m_vFrom.y;
			}
			Start.m_x = lpDoor1->m_vTo.x + 4;
			sParent = m_iHorzStrip;
		}
		else if(bDir == 1)
		{	// north/south
			if (Side==1)
			{
				Line[0].m_vFrom.x = -2176;
				Line[0].m_vFrom.y = m_yNextDoor;
				Line[0].m_vTo.x = -2208;
				Line[0].m_vTo.y = m_yNextDoor;
				CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
				Line[1].m_vTo.x = -2208;
				Line[1].m_vTo.y = m_yNextDoor - 8;
				CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
				Line[2].m_vTo.x = -2176;
				Line[2].m_vTo.y = m_yNextDoor - 8;
			}
			else
			{
				Line[0].m_vFrom.x = -2144;
				Line[0].m_vFrom.y = m_yNextDoor;
				Line[0].m_vTo.x = -2176;
				Line[0].m_vTo.y = m_yNextDoor;
				CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
				Line[1].m_vTo.x = -2176;
				Line[1].m_vTo.y = m_yNextDoor - 8;
				CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
				Line[2].m_vTo.x = -2144;
				Line[2].m_vTo.y = m_yNextDoor - 8;
			}
			m_yNextDoor -= 32;

			if (Side==1)
			{
				Anchor.m_x = Line[2].m_vFrom.x;
				Start.m_x = lpDoor1->m_vTo.x;
			}
			else
			{
				Anchor.m_x = Line[2].m_vTo.x;
				Start.m_x = lpDoor1->m_vFrom.x;
			}
			Anchor.m_y = Line[1].m_vFrom.y - 4;
			Start.m_y = lpDoor1->m_vTo.y - 4;
			sParent = m_iVertStrip;
		}

		CopyMemory(&Line[3].m_vFrom,&Line[2].m_vTo,sizeof(VERTEX));
		CopyMemory(&Line[3].m_vTo,&Line[0].m_vFrom,sizeof(VERTEX));

		//SetFrontDoor(iType,Line[0].m_sRight.szMiddle, bDir);
		//SetFrontDoor(iType,Line[2].m_sRight.szMiddle, bDir);

		switch(iWallType)
		{
		case 91:	//Grey/Blue N/S
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD01L");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD01L");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD01U");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD01U");
			}
			break;
		case 95:	//Elevator N/S
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD01L");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD01L");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD01U");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD01U");
			}
			break;
		case 90:	//Grey/Blue E/W
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD01L1");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD01L1");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD01U1");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD01U1");
			}
			break;
		case 94:	//Elevator E/W
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD03L1");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD03L1");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD03U1");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD03U1");
			}
			break;
		case 93:	//Blue/Blue N/S
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD05L");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD05L");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD05U");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD05U");
			}
			break;
		case 97:	//Blue/Red N/S
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD06L");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD06L");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD06U");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD06U");
			}
			break;
		case 101:	//1-Way \/
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[2].m_sRight.szMiddle,"NZUD2");
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD07L");
			}
			else
			{
				lstrcpy(Line[2].m_sRight.szMiddle,"NZUD2");
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD07U");
			}
			break;
		case 99:	//1-Way ^
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD07L");
				lstrcpy(Line[0].m_sRight.szMiddle,"NZUD2");
			}
			else
			{
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD07U");
				lstrcpy(Line[0].m_sRight.szMiddle,"NZUD2");
			}
			break;
		case 92:	//Blue/Blue E/W
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD05L1");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD05L1");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD05U1");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD05U1");
			}
			break;
		case 96:	//Blue/Red E/W
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD06L1");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD06L1");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD06U1");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD06U1");
			}
			break;
		case 98:	//1-Way <
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD07L1");
				lstrcpy(Line[0].m_sRight.szMiddle,"NZUD21");
			}
			else
			{
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD07U1");
				lstrcpy(Line[0].m_sRight.szMiddle,"NZUD21");
			}
			break;
		case 100:	//1-Way >
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZUD21");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD07L1");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZUD21");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD07U1");
			}
			break;
		case 103:	//Brown N/S
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD08L");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD08L");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD08U");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD08U");
			}
			break;
		case 102:	//Brown E/W
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD08L1");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD08L1");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD08U1");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD08U1");
			}
			break;
		case 89:	//White Door N/S
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD02L");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD02L");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD02U");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD02U");
			}
			break;
		case 88:	//White Door E/W
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD02L1");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD02L1");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD02U1");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD02U1");
			}
			break;
		case 105:	//Grey/Grey N/S
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD04L");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD04L1");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD04U");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD04U");
			}
			break;
		case 104:	//Grey/Grey E/W
			if(iType > 1 && iType < 6)
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD04L1");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD04L1");
			}
			else
			{
				lstrcpy(Line[0].m_sRight.szMiddle,"NZD04U1");
				lstrcpy(Line[2].m_sRight.szMiddle,"NZD04U1");
			}
			break;
		default:
			lstrcpy(Line[0].m_sRight.szMiddle,"NZUD2");
			lstrcpy(Line[2].m_sRight.szMiddle,"NZUD2");
			break;
		}

		//lstrcpy(Line[0].m_sRight.szMiddle,"NZUD1");
		//lstrcpy(Line[2].m_sRight.szMiddle,"NZUD1");
		if(Side==0)
			Line[0].m_sRight.xOffset += 32;
		else
			Line[2].m_sRight.xOffset -= 32;

		CopyMemory(Line[1].m_sRight.szMiddle,"DOORTRAK",8);
		CopyMemory(Line[3].m_sRight.szMiddle,"DOORTRAK",8);
		Line[0].m_Special = 226;
		Line[0].m_Flags = 1028; //1540;
		Line[0].m_Args[0] = 1;
		if (Side==0)
			Line[0].m_Args[2] = iPoly;
		else
			Line[0].m_Args[2] = iPoly - 1;
		Line[0].m_Args[3] = bDir + 1;
		if (iType == DOOR_ONEWAYN || iType == DOOR_ONEWAYW)
			Line[0].m_Args[4] = 99;
		else
			Line[0].m_Args[4] = iType -1;
		Line[2].m_Special = 226;
		Line[2].m_Flags = 1028; //1540;
		Line[2].m_Args[0] = 1;
		if (Side==0)
			Line[2].m_Args[2] = iPoly;
		else
			Line[2].m_Args[2] = iPoly - 1;
		Line[2].m_Args[3] = bDir + 1;
		if (iType == DOOR_ONEWAYS || iType == DOOR_ONEWAYE)
			Line[2].m_Args[4] = 99;
		else
			Line[2].m_Args[4] = iType - 1;

		Line[3].m_Special = 1;
		Line[3].m_Args[0] = iPoly;
		if (Side==0)
			Line[3].m_Args[1] = iPoly + 1;

		for(i = 0; i < 4; i++)
		{
			lstrcpy(Line[i].m_sRight.szLower,"-");
			lstrcpy(Line[i].m_sRight.szUpper,"-");
			//if((i & 1) == 0)
				Line[i].m_Flags = 9729;
			//else
			//	Line[i].m_Flags = 1;
			Line[i].m_sRight.Sector = sParent;
			Line[i].m_sLeft.Sector = -1;
			AddLine(&Line[i]);
		}

		Anchor.m_Angle = iPoly;
		Anchor.m_Options = 2023;
		Anchor.m_Type = 9300;
		AddThing(&Anchor);

		Start.m_Angle = iPoly;
		Start.m_Options = 2023;
		Start.m_Type = 9301;
		AddThing(&Start);	
	}
}

VOID CMapConvert::BuildHoldingArea (VOID)
{
	int i;
	if(m_iConversionType == CTYPE_ZDOOM || m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
	{
		CMapLine Line[16];
		CMapThing TeleportDest;

		SECTOR Sector;

		Line[0].m_vFrom.x = -2214;
		Line[0].m_vFrom.y = 2214;
		Line[0].m_vTo.x = -2138;
		Line[0].m_vTo.y = 2214;
		CopyMemory(&Line[1].m_vFrom,&Line[0].m_vTo,sizeof(VERTEX));
		Line[1].m_vTo.x = -2138;
		Line[1].m_vTo.y = 2138;
		CopyMemory(&Line[2].m_vFrom,&Line[1].m_vTo,sizeof(VERTEX));
		Line[2].m_vTo.x = -2214;
		Line[2].m_vTo.y = 2138;
		CopyMemory(&Line[3].m_vFrom,&Line[2].m_vTo,sizeof(VERTEX));
		CopyMemory(&Line[3].m_vTo,&Line[0].m_vFrom,sizeof(VERTEX));

		Line[4].m_vFrom.x = -2208;
		Line[4].m_vFrom.y = 2208;
		Line[4].m_vTo.x = -2144;
		Line[4].m_vTo.y = 2208;
		CopyMemory(&Line[5].m_vFrom,&Line[4].m_vTo,sizeof(VERTEX));
		Line[5].m_vTo.x = -2144;
		Line[5].m_vTo.y = 2144;
		CopyMemory(&Line[6].m_vFrom,&Line[5].m_vTo,sizeof(VERTEX));
		Line[6].m_vTo.x = -2208;
		Line[6].m_vTo.y = 2144;
		CopyMemory(&Line[7].m_vFrom,&Line[6].m_vTo,sizeof(VERTEX));
		CopyMemory(&Line[7].m_vTo,&Line[4].m_vFrom,sizeof(VERTEX));

		Line[8].m_vFrom.x = -2194;
		Line[8].m_vFrom.y = 2194;
		Line[8].m_vTo.x = -2158;
		Line[8].m_vTo.y = 2194;
		CopyMemory(&Line[9].m_vFrom,&Line[8].m_vTo,sizeof(VERTEX));
		Line[9].m_vTo.x = -2158;
		Line[9].m_vTo.y = 2158;
		CopyMemory(&Line[10].m_vFrom,&Line[9].m_vTo,sizeof(VERTEX));
		Line[10].m_vTo.x = -2194;
		Line[10].m_vTo.y = 2158;
		CopyMemory(&Line[11].m_vFrom,&Line[10].m_vTo,sizeof(VERTEX));
		CopyMemory(&Line[11].m_vTo,&Line[8].m_vFrom,sizeof(VERTEX));

		Line[12].m_vFrom.x = -2192;
		Line[12].m_vFrom.y = 2192;
		Line[12].m_vTo.x = -2160;
		Line[12].m_vTo.y = 2192;
		CopyMemory(&Line[13].m_vFrom,&Line[12].m_vTo,sizeof(VERTEX));
		Line[13].m_vTo.x = -2160;
		Line[13].m_vTo.y = 2160;
		CopyMemory(&Line[14].m_vFrom,&Line[13].m_vTo,sizeof(VERTEX));
		Line[14].m_vTo.x = -2192;
		Line[14].m_vTo.y = 2160;
		CopyMemory(&Line[15].m_vFrom,&Line[14].m_vTo,sizeof(VERTEX));
		CopyMemory(&Line[15].m_vTo,&Line[12].m_vFrom,sizeof(VERTEX));

		FillSector(&Sector);
		Sector.Ceiling = 64;

		for(i = 0; i < 9; i+=4)
		{
			Line[i].m_Args[0] = 29;
		}
		for(i = 3; i < 12; i+=4)
		{
			Line[i].m_Args[0] = 29;
		}

		for(i = 1; i < 10; i+=4)
		{
			Line[i].m_Args[0] = 30;
		}
		for(i = 2; i < 11; i+=4)
		{
			Line[i].m_Args[0] = 30;
		}

		for(i = 0; i < 4; i++)
		{
			lstrcpy(Line[i].m_sRight.szLower,"-");
			lstrcpy(Line[i].m_sRight.szMiddle,"FLOOR");
			lstrcpy(Line[i].m_sRight.szUpper,"-");
			lstrcpy(Line[i].m_sLeft.szUpper,"-");
			lstrcpy(Line[i].m_sLeft.szMiddle,"-");
			lstrcpy(Line[i].m_sLeft.szLower,"-");
			Line[i].m_Flags = 1;
			Line[i].m_sRight.Sector = m_iNextSector;
			Line[i].m_sLeft.Sector = -1;
			Line[i].m_Special = 80;
			Line[i].m_Flags = 3584;
			AddLine(&Line[i]);
		}

		for(i = 4; i < 8; i++)
		{
			lstrcpy(Line[i].m_sRight.szUpper,"-");
			lstrcpy(Line[i].m_sRight.szMiddle,"-");
			lstrcpy(Line[i].m_sRight.szLower,"-");
			lstrcpy(Line[i].m_sLeft.szUpper,"-");
			lstrcpy(Line[i].m_sLeft.szMiddle,"-");
			lstrcpy(Line[i].m_sLeft.szLower,"-");
			Line[i].m_sRight.Sector = m_iNextSector;
			Line[i].m_sLeft.Sector = m_iNextSector;
			Line[i].m_Special = 80;
			Line[i].m_Flags = 1540;
			AddLine(&Line[i]);
		}

		for(i = 8; i < 12; i++)
		{
			lstrcpy(Line[i].m_sRight.szUpper,"-");
			lstrcpy(Line[i].m_sRight.szMiddle,"-");
			lstrcpy(Line[i].m_sRight.szLower,"-");
			lstrcpy(Line[i].m_sLeft.szUpper,"-");
			lstrcpy(Line[i].m_sLeft.szMiddle,"-");
			lstrcpy(Line[i].m_sLeft.szLower,"-");
			Line[i].m_sRight.Sector = m_iNextSector;
			Line[i].m_sLeft.Sector = m_iNextSector;
			Line[i].m_Special = 80;
			Line[i].m_Flags = 4613;
			AddLine(&Line[i]);
		}

		for(i = 12; i < 16; i++)
		{
			lstrcpy(Line[i].m_sRight.szUpper,"-");
			lstrcpy(Line[i].m_sRight.szMiddle,"-");
			lstrcpy(Line[i].m_sRight.szLower,"-");
			lstrcpy(Line[i].m_sLeft.szUpper,"-");
			lstrcpy(Line[i].m_sLeft.szMiddle,"-");
			lstrcpy(Line[i].m_sLeft.szLower,"-");
			Line[i].m_sRight.Sector = m_iNextSector;
			Line[i].m_sLeft.Sector = m_iNextSector;
			Line[i].m_Special = 121;
			Line[i].m_Args[0] = 1;
			Line[i].m_Flags = 5;
			AddLine(&Line[i]);
		}

		m_iNextSector++;
		AddSector(&Sector);

		TeleportDest.m_Type = 14;
		TeleportDest.m_Options |= 2023;
		TeleportDest.m_Angle = 90;
		TeleportDest.m_ID = 8;
		TeleportDest.m_Args[0] = 0;
		TeleportDest.m_Args[1] = 0;
		TeleportDest.m_Args[2] = 0;
		TeleportDest.m_Args[3] = 0;
		TeleportDest.m_Args[4] = 0;
		TeleportDest.m_x = -2176;
		TeleportDest.m_y = 2176;
		TeleportDest.m_z = 0;
		AddThing(&TeleportDest);
		}
}

VOID CMapConvert::BuildSectors (USHORT* lpFloor)
{
	USHORT* lpBuild = new USHORT[4096];
	USHORT* lpRow = lpBuild;
	INT x, y;
	SECTOR Sector;
	ZeroMemory(m_lpSectorTable,sizeof(USHORT) * 4096);
	CopyMemory(lpBuild,lpFloor,sizeof(USHORT) * 4096);
	m_lpAssign = lpBuild;
	for(y = 0; y < 64; y++)
	{
		for(x = 0; x < 64; x++)
		{
			if((lpRow[x] >= 106 || (m_bAllowFloorZero && lpRow[x] == 0)) && lpRow[x] != 65535)
			{
				FillSector(&Sector);
				AddSector(&Sector);
				AssignSector(x,y);
				m_iNextSector++;
			}
		}
		lpRow += 64;
	}
	m_lpAssign = NULL;
	delete [] lpBuild;
}

VOID CMapConvert::BuildWalkover (USHORT* lpFloor, USHORT* lpObject)
{
	INT x, y;
	USHORT* lpRow = lpObject;
	SECTOR Sector;
	for(y = 0; y < 64; y++)
	{
		for(x = 0; x < 64; x++)
		{
			if(lpRow[x] == 99)
			{
				AssignWalkover(x,y,lpObject);
				FillSector(&Sector);
				AddSector(&Sector);
				m_iNextSector++;
			}
		}
		lpRow += 64;
	}
}

VOID CMapConvert::AssignWalkover (INT x, INT y, USHORT* lpObject)
{
	LONG iPos = y * 64 + x;
	if(lpObject[iPos] == 99)
	{
		CMapLine Line, *lpPtr;
		LONG iDir, index;
		lpObject[iPos] += 1000;
		for(iDir = LINE_LEFT; iDir <= LINE_BELOW; iDir++)
		{
			switch(iDir)
			{
			case LINE_LEFT:
				index = iPos - 1;
				break;
			case LINE_RIGHT:
				index = iPos + 1;
				break;
			case LINE_ABOVE:
				index = iPos + 64;
				break;
			case LINE_BELOW:
				index = iPos - 64;
				break;
			}
			if(lpObject[index] == 99)
				AssignWalkover(index % 64,index / 64,lpObject);
			else
			{
				PositionLine(&Line,x,y,iDir);
				lpPtr = FindLine(&Line.m_vFrom,&Line.m_vTo);
				if(lpPtr)
				{
					BOOL bRemove;
					if(m_iConversionType == CTYPE_DOOM || m_iConversionType == CTYPE_EDGE)
						bRemove = (lpPtr->m_Types == 52);
					else if(m_iConversionType == CTYPE_ZDOOM)
						bRemove = (lpPtr->m_Special == 75);
					//TJL - Added other conversion type to list...
					else if(m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
						bRemove = (lpPtr->m_Special == 80);

					if(bRemove)
						RemoveLine(lpPtr);
					else
						lpPtr->m_sRight.Sector = m_iNextSector;
				}
				else
				{
					FlipLinedef(&Line);
					lpPtr = FindLine(&Line.m_vFrom,&Line.m_vTo);
					if(lpPtr)
					{
						lpPtr->m_sLeft.Sector = m_iNextSector;
						if(m_iConversionType == CTYPE_DOOM || m_iConversionType == CTYPE_EDGE)
							lpPtr->m_Types = 52;
						//TJL - Added other conversion types to list...
						else if(m_iConversionType == CTYPE_ZDOOM)
							lpPtr->m_Special = 75;
						else if(m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
						{
							lpPtr->m_Special = 80;
							lpPtr->m_Args[0] = 4;
							lpPtr->m_Args[1] = 0;
							lpPtr->m_Args[2] = 1;
							lpPtr->m_Args[3] = 0;
							lpPtr->m_Args[4] = 0;
						}
					}
					else if(lpObject[index] != 1099)
					{
						Line.m_Flags = 4;
						if(m_iConversionType == CTYPE_DOOM || m_iConversionType == CTYPE_EDGE)
							Line.m_Types = 52;
						//TJL - Added other conversion types to list...
						else if(m_iConversionType == CTYPE_ZDOOM)
							Line.m_Special = 75;
						else if(m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
						{
							Line.m_Special = 80;
							Line.m_Args[0] = 4;
							Line.m_Args[1] = 0;
							Line.m_Args[2] = 1;
							Line.m_Args[3] = 0;
							Line.m_Args[4] = 0;
						}
						Line.m_sRight.Sector = FindSector(x,y);
						Line.m_sLeft.Sector = m_iNextSector;

						lstrcpy(Line.m_sRight.szUpper,"-");
						lstrcpy(Line.m_sRight.szMiddle,"-");
						lstrcpy(Line.m_sRight.szLower,"-");
						lstrcpy(Line.m_sLeft.szUpper,"-");
						lstrcpy(Line.m_sLeft.szMiddle,"-");
						lstrcpy(Line.m_sLeft.szLower,"-");

						AddLine(&Line);
					}
				}
			}
		}
		m_lpSectorTable[y * 64 + x] = m_iNextSector;
	}
}

VOID CMapConvert::BuildThings (USHORT* lpObject, USHORT* lpFloor)
{
	BOOL bDeaf;
	INT x, y;
	USHORT iType, f;
	CMapThing Thing;
	Thing.m_Options = 7;
	for(y = 0; y < 64; y++)
	{
		for(x = 0; x < 64; x++)
		{
			iType = lpObject[x];
			//if(iType != 0)
			{
				f = lpFloor[x];
				bDeaf = (f == 106) ? TRUE : FALSE;
				if(m_bSimpleConversion)
					ConvertThing_Simple(&Thing,x,y,iType,bDeaf);
				else
				{
					if(f == 229)		// End of Map
						iType = 99;
					else if ((f == 108 || f == 162) && m_iConversionType == CTYPE_BSTC)	//Guard Block
					{
						CMapThing MonsterBlock;
						MonsterBlock = Thing;
						MonsterBlock.m_Type = 25000;
						MonsterBlock.m_Options |= 1792;
						PositionThing(&MonsterBlock,x,y);
						AddThing(&MonsterBlock);
					}

					if (bDeaf)
					{
						if ((lpFloor[x+1] < 143) && (108 < lpFloor[x+1]))
							f = lpFloor[x+1];
						else if ((lpFloor[x-1] < 143) && (108 < lpFloor[x-1]))
							f = lpFloor[x-1];
						else if ((lpFloor[x+64] < 143) && (108 < lpFloor[x+64]))
							f = lpFloor[x+64];
						else if ((lpFloor[x-64] < 143) && (108 < lpFloor[x-64]))
							f = lpFloor[x-64];
						else
							f = 0;
						f += 500;
					}
					if(f < 88 || f > 105 || f == 108 || f == 162) // || iType == 98)
						ConvertThing_TotalConversion(lpObject,&Thing,x,y,iType,bDeaf,f);
				}
			}
		}
		lpObject += 64;
		lpFloor += 64;
	}
}

INT CMapConvert::CheckThingPos (INT x, INT y, INT dx, INT dy, USHORT* lpObject, USHORT* lpFloor)
{
	int xloop = 0;
	int yloop = 0;
	while ((lpFloor[y * 64 + x] < 101) || (lpObject[y * 64 + x]))
	{
		xloop++;
		if (xloop >= 64)
			y += 1;

		if (x < 1)
			x = 64;
		else if (x <= 64)
			x += dx;
		else
			x = 1;
		
		yloop++;
		if (yloop >= 64)
			x += 1;

		if (y < 1)
			y = 64;
		else if (y <= 64)
			y += dy;
		else
			y = 1;
	}
	return (y * 64 + x);
}

VOID CMapConvert::BuildPlayerStarts (USHORT* lpObject, USHORT* lpFloor)
{
	INT x, y, c, d, x2, y2, dx, dy;
	USHORT iType, f;
	CMapThing Thing;

	Thing.m_Options = 7;
	for(y = 0; y < 64; y++)
	{
		for(x = 0; x < 64; x++)
		{
			iType = lpObject[y * 64 + x];
			if(iType != 0)
			{
				f = lpFloor[y * 64 + x];
				if(iType >= 19 && iType <= 22)
				{						// Player start!
					SHORT Angle[] = {90,0,270,180};
					Thing.m_Angle = Angle[iType - 19];

					switch (Angle[iType - 19])
					{
					case 90:
						y2 += -1;
						for (c = 2; c <= 4; c++)
						{
							y2 = y;
							x2 = x;
							dx = 0;
							dy = 0;

							Thing.m_Type = c;
							Thing.m_Options |= 775;

							switch (c)
							{
							case 2:
								x2 += -1;
								dx = -1;
								break;
							case 3:
								y2 += -1;
								dy = -1;
								break;
							case 4:
								x2 += 1;
								dx = 1;
								break;
							}
							d = CheckThingPos(x2, y2, dx, dy, lpObject, lpFloor);
							x2 = d % 64;
							y2 = d / 64;

							PositionThing(&Thing,x2,y2);
							AddThing(&Thing);
						}
						break;
					case 0:
						x2 += -1;
						for (c = 2; c <= 4; c++)
						{
							y2 = y;
							x2 = x;
							dx = 0;
							dy = 0;

							Thing.m_Type = c;
							Thing.m_Options |= 775;

							switch (c)
							{
							case 2:
								y2 += 1;
								dy = 1;
								break;
							case 3:
								y2 += -1;
								dy = -1;
								break;
							case 4:
								x2 += 1;
								dx = 1;
								break;
							}
							d = CheckThingPos(x2, y2, dx, dy, lpObject, lpFloor);
							x2 = d % 64;
							y2 = d / 64;

							PositionThing(&Thing,x2,y2);
							AddThing(&Thing);
						}
						break;
					case 270:
						y2 += 1;
						for (c = 2; c <= 4; c++)
						{
							y2 = y;
							x2 = x;
							dx = 0;
							dy = 0;

							Thing.m_Type = c;
							Thing.m_Options |= 775;

							switch (c)
							{
							case 2:
								x2 += -1;
								dx = -1;
								break;
							case 3:
								y2 += 1;
								dy = 1;
								break;
							case 4:
								x2 += 1;
								dx = 1;
								break;
							}
							d = CheckThingPos(x2, y2, dx, dy, lpObject, lpFloor);
							
							x2 = d % 64;
							y2 = d / 64;

							PositionThing(&Thing,x2,y2);
							AddThing(&Thing);
						}

						break;
					case 180:
						x2 += 1;
						for (c = 2; c <= 4; c++)
						{
							y2 = y;
							x2 = x;
							dx = 0;
							dy = 0;

							Thing.m_Type = c;
							Thing.m_Options |= 775;

							switch (c)
							{
							case 2:
								y2 += -1;
								dy = -1;
								break;
							case 3:
								y2 += 1;
								dy = 1;
								break;
							case 4:
								x2 += -1;
								dx = -1;
								break;
							}
							d = CheckThingPos(x2, y2, dx, dy, lpObject, lpFloor);
							x2 = d % 64;
							y2 = d / 64;

							PositionThing(&Thing,x2,y2);
							AddThing(&Thing);
						}
						break;
					}
				}
			}
		}
	}
}

VOID CMapConvert::BuildDMPlayerStarts (USHORT* lpObject, USHORT* lpFloor)
{
	INT x = 0;
	CMapThing Thing;

	SHORT Angle[] = {90,0,270,180};

	Thing.m_Options = 7;
	Thing.m_Type = 11;
	Thing.m_Options |= 775;

	for (int l = 1; l <= 8; l++)
	{
		Thing.m_Angle = Angle[rand() % 3];
		while (lpFloor[x] < 101 || lpObject[x])
		{
			x = rand() % 4096 + 1;
		}
		PositionThing(&Thing,x % 64,x / 64);
		AddThing(&Thing);
		x = 0;
	}
}

VOID CMapConvert::ConvertThing_Simple (CMapThing* lpThing, INT x, INT y, USHORT iType, BOOL bDeaf)
{
	BOOL bAdd = TRUE;
	if(iType == 43)			// Gold key
		lpThing->m_Type = 6;
	else if(iType == 44)	// Silver key
		lpThing->m_Type = 5;
	else if(iType == 47)	// Dinner
		lpThing->m_Type = 2011;
	else if(iType == 48)	// First aid
		lpThing->m_Type = 2012;
	else if(iType == 49)	// Ammo
		lpThing->m_Type = 2007;
	else if(iType == 50)	// Machine gun
		lpThing->m_Type = 2001;
	else if(iType == 51)	// Chaingun
		lpThing->m_Type = 2002;
	else if(iType == 52)	// Cross
		lpThing->m_Type = 2008;
	else if(iType == 56)	// Extra life
		lpThing->m_Type = 2013;
	else if(iType == 72)	// Box of ammo
		lpThing->m_Type = 2048;
	else if(iType == 124)	// Dead guard
		lpThing->m_Type = 18;
	else if(iType == 197)	// Grettel Grosse
	{
		ConvertThing_Simple(lpThing,x,y,43,FALSE);
		lpThing->m_Type = 3005;
		lpThing->m_Angle = 90;
		bAdd = TranslateActor(lpThing,iType);
	}
	else if(iType == 214)	// Hans Grosse
	{
		ConvertThing_Simple(lpThing,x,y,43,FALSE);
		lpThing->m_Type = 3005;
		lpThing->m_Angle = 270;
		bAdd = TranslateActor(lpThing,iType);
	}
	else if(iType >= 19 && iType <= 22)
	{						// Player start!
		SHORT Angle[] = {90,0,270,180};
		lpThing->m_Type = 1;
		lpThing->m_Angle = Angle[iType - 19];
	}
	else
	{
		if(CPrefabs::ConvertActor(lpThing,iType))
			bAdd = TranslateActor(lpThing,iType);
		else
		{
			lpThing->m_Type = 0;	// Can only be translated by a super class
			bAdd = TranslateActor(lpThing,iType);
		}
	}
	if(bAdd)
	{
		if(bDeaf)
			lpThing->m_Options |= 8;
		PositionThing(lpThing,x,y);
		if(m_iConversionType == CTYPE_ZDOOM || m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
			lpThing->m_Options |= 256;
		AddThing(lpThing);
		lpThing->m_Angle = 0;
		lpThing->m_Options = 7;
	}
}

VOID CMapConvert::ConvertThing_TotalConversion (USHORT* lpObject, CMapThing* lpThing, INT x, INT y, USHORT iType, BOOL bDeaf, INT iFloor)
{
	BOOL bAdd = TRUE;

	//TJL - New codes for AFADoomer's TC items
	if(m_iConversionType != CTYPE_BSTC)
	{
		if(iType == 23)					//  Water
			lpThing->m_Type = 21023;
		else if(iType == 24)			//  Oil drum
			lpThing->m_Type = 21024;
		else if(iType == 25)			//  Table w/chairs
			lpThing->m_Type = 21025;
		else if(iType == 26)			//  Floor lamp
			lpThing->m_Type = 21026;
		else if(iType == 27)			//  Chandalier
			lpThing->m_Type = 21027;
		else if(iType == 28)			//  Hanging Skeleton
			lpThing->m_Type = 21028;
		else if(iType == 29)			//  Dog food
			lpThing->m_Type = 21029;
		else if(iType == 30)			//  Column
			lpThing->m_Type = 21030;
		else if(iType == 31)			//  Plant
			lpThing->m_Type = 21031;
		else if(iType == 32)			//  Skeleton
			lpThing->m_Type = 21032;
		else if(iType == 33)			//  Sink		//  Human Skulls
			lpThing->m_Type = 21033;
		else if(iType == 34)			//  Plant in vase
			lpThing->m_Type = 21034;
		else if(iType == 35)			//  Vase
			lpThing->m_Type = 21035;
		else if(iType == 36)			//  Table
			lpThing->m_Type = 21036;
		else if(iType == 37)			//  Green Ceiling light
			lpThing->m_Type = 21037;
		else if(iType == 38)			//  Kitchen utensils	//  Bloody Cage
			lpThing->m_Type = 21038;
		else if(iType == 39)			//  Armor
			lpThing->m_Type = 21039;
		else if(iType == 40)			//  Empty cage
			lpThing->m_Type = 21040;
		else if(iType == 41)			//  Cage w/skeleton
			lpThing->m_Type = 21041;
		else if(iType == 42)			//  Bones
			lpThing->m_Type = 21042;
		else if(iType == 43)			//  Yellow key
			lpThing->m_Type = 21043;
		else if(iType == 44)			//  Blue Key
			lpThing->m_Type = 21044;
		else if(iType == 45)			//  Bed			//  Skull Cage
			lpThing->m_Type = 21045;
		else if(iType == 46)			//  Bin
			lpThing->m_Type = 21046;
		else if(iType == 47)			//  Food
			lpThing->m_Type = 21047;
		else if(iType == 48)			//  First aid kit
			lpThing->m_Type = 21048;
		else if(iType == 49)			//  Ammunition
			lpThing->m_Type = 21049;
		else if(iType == 50)			//  Machine gun
			lpThing->m_Type = 21050;
		else if(iType == 51)			//  Chain gun
			lpThing->m_Type = 21051;
		else if(iType == 52)			//  Cross
			lpThing->m_Type = 21052;
		else if(iType == 53)			//  Chalace
			lpThing->m_Type = 21053;
		else if(iType == 54)			//  Jewels
			lpThing->m_Type = 21054;
		else if(iType == 55)			//  Crown
			lpThing->m_Type = 21055;
		else if(iType == 56)			//  Extra Life
			lpThing->m_Type = 21056;
		else if(iType == 57)			//  Bones w/blood
			lpThing->m_Type = 21057;
		else if(iType == 58)			//  Barrel
			lpThing->m_Type = 21058;
		else if(iType == 59)			//  Well w/water
			lpThing->m_Type = 21059;
		else if(iType == 60)			//  Well wo/water
			lpThing->m_Type = 21060;
		else if(iType == 61)			//  Pool of blood
			lpThing->m_Type = 21061;
		else if(iType == 62)			//  Flag
			lpThing->m_Type = 21062;
		else if(iType == 63)			//  `Aardwolf' sign	//  Red Ceiling Light
			lpThing->m_Type = 21063;
		else if(iType == 64)			//  Crushed Bones 1
			lpThing->m_Type = 21064;
		else if(iType == 65)			//  Crushed Bones 2
			lpThing->m_Type = 21065;
		else if(iType == 66)			//  Crushed Body
			lpThing->m_Type = 21066;
		else if(iType == 67)			//  Hanging utensils 2	//  Cow Skull
			lpThing->m_Type = 21067;
		else if(iType == 68)			//  Stove		//  Well w/Blood
			lpThing->m_Type = 21068;
		else if(iType == 69)			//  Spear Rack		//  Angel Statue
			lpThing->m_Type = 21069;
		else if(iType == 70)			//  Hanging Vines
			lpThing->m_Type = 21070;
		else if(iType == 124)			//  Dead Guard
			lpThing->m_Type = 21124;

		//SOD Stuff
		else if(iType == 71)			//  Brown Column
			lpThing->m_Type = 21071;
		else if(iType == 72)			//  25 Bullets
			lpThing->m_Type = 21072;
		else if(iType == 73)			//  Truck Rear
			lpThing->m_Type = 21073;
		else if(iType == 74)			//  Spear
			lpThing->m_Type = 21074;
		else if(iType == 106)			//  GHOST
			lpThing->m_Type = 20007;
		else if(iType == 125)			//  Trans Grosse
			lpThing->m_Type = 20015;
		else if(iType == 142)			//  UberMutant
			lpThing->m_Type = 20017;
		else if(iType == 143)			//  Barnacle Wilhelm
			lpThing->m_Type = 20016;
		else if(iType == 161)			//  Death Knight
			lpThing->m_Type = 20018;
		else if(iType == 107)			//  Angel of Death
			lpThing->m_Type = 20019;

		//Bosses
		else if(iType == 214)			//  Hans Grosse
		{
			lpThing->m_Type = 20010;
			lpThing->m_Angle = 270;
		}
		else if(iType == 197)			//  Gretel Grosse
		{
			lpThing->m_Type = 20014;
			lpThing->m_Angle = 270;
		}
		else if(iType == 178)			//  Hitler
		{
			lpThing->m_Type = 20012;
			lpThing->m_Angle = 270;
		}
		else if(iType == 179)			//  General Fettgesicht
		{
			lpThing->m_Type = 20015;
			lpThing->m_Angle = 270;
		}
		else if(iType == 160)			//  Hitler Ghost
		{
			lpThing->m_Type = 20006;
			lpThing->m_Angle = 180;
		}
		else if(iType == 196)			//  Dr. Schabbs
		{
			lpThing->m_Type = 20011;
			lpThing->m_Angle = 270;
		}
		else if(iType == 215)			//  Otto Giftmacher
		{
			lpThing->m_Type = 20013;
			lpThing->m_Angle = 270;
		}
		else if(iType == 224)			//  Red Ghost
			lpThing->m_Type = 20020;
		else if(iType == 225)			//  Yellow Ghost
			lpThing->m_Type = 20021;
		else if(iType == 226)			//  Green Ghost (Pink)
			lpThing->m_Type = 20022;
		else if(iType == 227)			//  Blue Ghost
			lpThing->m_Type = 20023;
		else if(iType >= 19 && iType <= 22)
		{						// Player start!
			SHORT Angle[] = {90,0,270,180};
			lpThing->m_Type = 1;
			lpThing->m_Angle = Angle[iType - 19];
		}
		else if(iType == 98)			//  Secret Door
			lpThing->m_Type = 22100;
		else
		{
			if(CPrefabs::ConvertActorTC(lpThing,iType))
			{
				bAdd = TranslateActor(lpThing,iType);
			}
			else
			{
				lpThing->m_Type = 0;	// Can only be translated by a super class
				bAdd = TranslateActor(lpThing,iType);
			}
		}
		lpThing->m_ID = iFloor; //Give enemy actors the original floor area code as TID
	}
	else
	{
		if(iType >= 19 && iType <= 22)
		{
			SHORT Angle[] = {90,0,270,180};
			lpThing->m_Type = 1;
			lpThing->m_Angle = Angle[iType - 19];
		}
		else if(iType > 0 && iType <= 70 || (iType >= 75 && iType <= 88) || iType == 160 || iType == 173 || (iType == 174 || iType == 175) || (iType >= 178 && iType <= 179) || (iType >= 381 && iType <= 485))
		{
			lpThing->m_Type = 21000 + iType;
		}
		else
		{
			if(CPrefabs::ConvertActorTC(lpThing,iType))
			{
				bAdd = TranslateActor(lpThing,iType);
			}
			else
			{
				lpThing->m_Type = 0;	// Can only be translated by a super class
				bAdd = TranslateActor(lpThing,iType);
			}
		}
		lpThing->m_ID = iFloor; //Give enemy actors the original floor area code as TID
		if(iType == 174 || iType == 175)
			lpThing->m_ID = x*256 + y; //CheckThingNeighbor(lpObject, iType, x, y, 0);
	}

	if(bAdd)
	{
		if(bDeaf) //Deaf
			lpThing->m_Options |= 8;
		PositionThing(lpThing,x,y);
		if(m_iConversionType == CTYPE_ZDOOM || m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
			lpThing->m_Options |= 1792;
		AddThing(lpThing);
		lpThing->m_Angle = 0;
		lpThing->m_Options = 7;
		lpThing->m_ID = 0;
	}
}

INT CMapConvert::CheckThingNeighbor (USHORT* lpObject, INT iType, INT x, INT y, INT from)
{
	USHORT* lpRow = lpObject;
	INT results[5] = {0,0,0,0,0}, thisitem, i, temp;

	thisitem = (256 * x) + y;

	results[0] = thisitem;

	if(lpRow[x - 1] == iType && from != 4)	//Left
		results[1] = CheckThingNeighbor(lpRow, iType, x - 1, y, 3);
	if(lpRow[x - 64] == iType && from != 1)	//Above
		results[2] = CheckThingNeighbor(lpRow-=64, iType, x, y - 1, 2);
	if(lpRow[x + 64] == iType && from != 2)	//Below
		results[3] = CheckThingNeighbor(lpRow+=64, iType, x, y + 1, 1);
	if(lpRow[x + 1] == iType && from != 3)	//Right
		results[4] = CheckThingNeighbor(lpRow, iType, x + 1, y, 4);

	temp = thisitem;

	for(i = 0; i < 5; i++)
	{
		if(results[i] > 0)
		{
			if(results[i] <= temp)
				temp = results[i];
		}
	}
	return temp;
}

BOOL CMapConvert::TranslateActor (CMapThing* lpThing, USHORT iWolfType)
{
	if(lpThing->m_Type == 0)
		return FALSE;
	return TRUE;
}

VOID CMapConvert::ConvertFinale (USHORT* lpObject)
{
	INT x, y, count = 0, mark = 0;
	USHORT* lpRow = lpObject;
	CMapThing Thing;
	for(y = 0; y < 64; y++)
	{
		for(x = 0; x < 64; x++)
		{
			if(lpRow[x] == 1099) //Changed from 99 in AddWalkover
			{
				count++;
				if (!mark)
					mark = x;
			}
		}
		if (count)
		{
			x = (mark * 64 - 2048) + (32 * count);
			y = (2048 - y * 64) - 64;
			Thing.m_Type = 20000;
			Thing.m_Options |= 1015;
			Thing.m_Angle = 90;
			Thing.m_ID = 1;
			Thing.m_Special = 229;
			Thing.m_Args[1] = 9;
			Thing.m_Args[2] = 1;
			Thing.m_x = x;
			Thing.m_y = y;
			AddThing(&Thing);

			Thing.Reset();
			Thing.m_Type = 9072;
			Thing.m_Options |= 2023;
			Thing.m_Angle = 90;
			Thing.m_ID = 2;
			Thing.m_Args[0] = 7;
			Thing.m_Args[1] = 0;
			Thing.m_Args[2] = 3;
			Thing.m_Args[3] = 0;
			Thing.m_Args[4] = 0;
			Thing.m_x = x;
			Thing.m_y = y - 96;
			Thing.m_z = 32;
			AddThing(&Thing);

			Thing.Reset();
			Thing.m_Type = 9070;
			Thing.m_Options |= 2023;
			Thing.m_Angle = 90;
			Thing.m_ID = 7;
			Thing.m_Args[0] = 0;
			Thing.m_Args[1] = 4;
			Thing.m_Args[2] = 0;
			Thing.m_Args[3] = 3;
			Thing.m_Args[4] = 0;
			Thing.m_x = x;
			Thing.m_y = y - 32;
			Thing.m_z = 32;
			AddThing(&Thing);

			Thing.Reset();
			Thing.m_Type = 9070;
			Thing.m_Options |= 2023;
			Thing.m_Angle = 90;
			Thing.m_ID = 3;
			Thing.m_Args[0] = 0;
			Thing.m_Args[1] = 2;
			Thing.m_Args[2] = 0;
			Thing.m_Args[3] = 4;
			Thing.m_Args[4] = 0;
			Thing.m_x = x;
			Thing.m_y = y + 32;
			Thing.m_z = 32;
			AddThing(&Thing);

			Thing.Reset();
			Thing.m_Type = 9070;
			Thing.m_Options |= 2023;
			Thing.m_Angle = 90;
			Thing.m_ID = 4;
			Thing.m_Args[0] = 0;
			Thing.m_Args[1] = 10;
			Thing.m_Args[2] = 0;
			Thing.m_Args[3] = 5;
			Thing.m_Args[4] = 0;
			Thing.m_x = x;
			Thing.m_y = y + 64;
			Thing.m_z = 32;
			AddThing(&Thing);

			Thing.Reset();
			Thing.m_Type = 9070;
			Thing.m_Options |= 2023;
			Thing.m_Angle = 270;
			Thing.m_ID = 5;
			Thing.m_Args[0] = 0;
			Thing.m_Args[1] = 6;
			Thing.m_Args[2] = 0;
			Thing.m_Args[3] = 6;
			Thing.m_Args[4] = 0;
			Thing.m_x = x;
			Thing.m_y = y + 128;
			Thing.m_z = 32;
			AddThing(&Thing);

			Thing.Reset();
			Thing.m_Type = 9070;
			Thing.m_Options |= 2023;
			Thing.m_Angle = 270;
			Thing.m_ID = 6;
			Thing.m_Args[0] = 0;
			Thing.m_Args[1] = 0;
			Thing.m_Args[2] = 0;
			Thing.m_Args[3] = 0;
			Thing.m_Args[4] = 0;
			Thing.m_x = x;
			Thing.m_y = y + 224;
			Thing.m_z = 32;
			AddThing(&Thing);

			Thing.Reset();
			Thing.m_Type = 9024;
			Thing.m_Options |= 2023;
			Thing.m_Angle = 90;
			Thing.m_ID = 9;
			Thing.m_Args[0] = 0;
			Thing.m_Args[1] = 0;
			Thing.m_Args[2] = 0;
			Thing.m_Args[3] = 0;
			Thing.m_Args[4] = 0;
			Thing.m_x = x;
			Thing.m_y = y + 224;
			Thing.m_z = 32;
			AddThing(&Thing);

			count = 0;
			mark = 0;
		}
		lpRow+=64;
	}
}

BOOL CMapConvert::AssignSector (INT x, INT y)
{
   INT i, index;
   CMapLine Line, *lpFind;
   USHORT f;
   if(x >= 0 && y >= 0 && x < 64 && y < 64)
      f = m_lpAssign[y * 64 + x];
   else
      f = 1;
   if((f >= 106 || (m_bAllowFloorZero && f == 0)) && f != 65535)
   {
      index = y * 64 + x;
      m_lpAssign[index] = 65535;
      m_lpSectorTable[index] = m_iNextSector;
      for(i = LINE_LEFT; i <= LINE_BELOW; i++)
      {
         PositionLine(&Line,x,y,i);
         lpFind = FindLine(&Line.m_vFrom,&Line.m_vTo);
         if(lpFind)
            lpFind->m_sRight.Sector = m_iNextSector;
      }
      AssignSector(x-1,y);
      AssignSector(x+1,y);
      AssignSector(x,y-1);
      AssignSector(x,y+1);
      return TRUE;
   }
   return FALSE;
}

VOID CMapConvert::MergeDone (CMapLine* lpLine)
{
}

VOID CMapConvert::MergeLines (VOID)
{
	LPLINE_LIST lpList, lpFind, lpPrev;
	CMapLine* lpLine, *lpTest;
	do
	{
		lpList = m_lpList;
		while(lpList)
		{
			lpLine = lpList->lpLine;
			lpFind = FindNext(&lpLine->m_vTo,&lpPrev);
			if(lpFind && (lpFind->lpLine->m_sRight.Sector == lpLine->m_sRight.Sector) && (lpFind->lpLine->m_sLeft.Sector == lpLine->m_sLeft.Sector))
			{
				BOOL bNext = FALSE;
				lpTest = lpFind->lpLine;
				if(lpLine->m_vFrom.x == lpLine->m_vTo.x)
				{
					if(lpLine->m_vTo.x == lpTest->m_vTo.x)
						bNext = TRUE;
				}
				else
				{
					if(lpLine->m_vTo.y == lpTest->m_vTo.y)
						bNext = TRUE;
				}
				if(bNext)
				{
					if(lstrcmp(lpLine->m_sRight.szMiddle,lpTest->m_sRight.szMiddle) == 0)
					{
						lpLine->m_vTo.x = lpTest->m_vTo.x;
						lpLine->m_vTo.y = lpTest->m_vTo.y;
						if(lpPrev)
							lpPrev->Next = lpFind->Next;
						else
							m_lpList = lpFind->Next;
						delete lpFind;
						MergeDone(lpLine);
						break;
					}
				}
			}
			lpList = lpList->Next;
		}
	} while(lpList);
}

VOID CMapConvert::BuildOutsideViews (VOID)
{
	LPLINE_LIST lpList = m_lpList;
	CMapLine* lpLine;
	while(lpList)
	{
		lpLine = lpList->lpLine;
		if(lstrcmp(lpLine->m_sRight.szMiddle,"NZ10") == 0)
		{
			INT iDir;
			if(lpLine->m_vFrom.x == lpLine->m_vTo.x)
			{
				if(lpLine->m_vFrom.y > lpLine->m_vTo.y)
					iDir = LINE_RIGHT;
				else
					iDir = LINE_LEFT;
			}
			else
			{
				if(lpLine->m_vFrom.x < lpLine->m_vTo.x)
					iDir = LINE_ABOVE;
				else
					iDir = LINE_BELOW;
			}
			CPrefabs::ExtendOutside(this,lpLine,iDir,"NZ01");
		}
		lpList = lpList->Next;
	}
}

VOID CMapConvert::BuildPrefabs (USHORT* lpObject)
{
	INT x, y;
	CMapThing Dummy;
	for(y = 0; y < 64; y++)
	{
		for(x = 0; x < 64; x++)
		{
			switch(lpObject[x])
			{
			case 24:		// Oil drum
				PositionThing(&Dummy,x,y);
				CPrefabs::CreateOilDrum(this,FindSector(x,y),Dummy.m_x,Dummy.m_y);
				break;
			case 30:		// Column
				PositionThing(&Dummy,x,y);
				CPrefabs::CreateColumn(this,FindSector(x,y),Dummy.m_x,Dummy.m_y,0);
				break;
			case 58:		// Barrel
				PositionThing(&Dummy,x,y);
				CPrefabs::CreateBarrel(this,FindSector(x,y),Dummy.m_x,Dummy.m_y);
				break;
			case 71:		// Brown column
				PositionThing(&Dummy,x,y);
				CPrefabs::CreateColumn(this,FindSector(x,y),Dummy.m_x,Dummy.m_y,1);
				break;
			case 74:		// The Spear
				PositionThing(&Dummy,x,y);
				CPrefabs::CreateSpear(this,FindSector(x,y),Dummy.m_x,Dummy.m_y);
				break;
			}
		}
		lpObject += 64;
	}
}

VOID CMapConvert::BuildPolys (VOID)
{
	if(m_iConversionType == CTYPE_ZDOOM || m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
	{
		LPPOLY_QUEUE lpPoly;
		CMapLine* lpDoor1, *lpDoor2;
		while(m_lpQueue)
		{
			lpPoly = m_lpQueue;
			m_lpQueue = m_lpQueue->Next;
			lpDoor1 = new CMapLine(&lpPoly->Door1);
			lpDoor2 = new CMapLine(&lpPoly->Door2);
			if(m_iConversionType == CTYPE_BSTC)
			{
				BuildPolyDoor(lpDoor1,lpDoor2,lpPoly->iType % 10,lpPoly->bDir,lpPoly->iTexture,lpPoly->iPoly, 0, lpPoly->iType/10);
				lpPoly = m_lpQueue;
				m_lpQueue = m_lpQueue->Next;
				lpDoor1 = new CMapLine(&lpPoly->Door1);
				lpDoor2 = new CMapLine(&lpPoly->Door2);
				BuildPolyDoor(lpDoor1,lpDoor2,lpPoly->iType % 10,lpPoly->bDir,lpPoly->iTexture,lpPoly->iPoly, 1, lpPoly->iType/10);
			}
			else
				BuildPolyDoor(lpDoor1,lpDoor2,lpPoly->iType,lpPoly->bDir,lpPoly->iTexture,lpPoly->iPoly, 0, 0);
			delete lpDoor2;
			delete lpDoor1;
			delete lpPoly;
		}
	}
}

VOID CMapConvert::FixSidedefs (VOID)
{
	LPLINE_LIST lpList = m_lpList;
	CMapLine* lpLine;
	while(lpList)
	{
		lpLine = lpList->lpLine;
		if(lpLine->m_Flags & 4)
		{
			if(lpLine->m_sRight.Sector == -1)
			{
				FlipLinedef(lpLine);
				lpLine->m_Flags &= ~4;
				lpLine->m_Flags |= 1;
				lstrcpy(lpLine->m_sRight.szMiddle,"NZ01");
			}
		}
		lpList = lpList->Next;
	}
}

VOID CMapConvert::BuildVertexList (VOID)
{
	LPLINE_LIST lpL = m_lpList;
	LPVERTEX_LIST lpV, lpPrev = NULL;
	LPVERTEX lpVertex;
	INT i;
	while(lpL)
	{
		for(i = 0; i < 2; i++)
		{
			if(i == 0)
				lpVertex = &lpL->lpLine->m_vFrom;
			else
				lpVertex = &lpL->lpLine->m_vTo;
			if(FindVertex(lpVertex) == 65535)
			{
				lpV = new VERTEX_LIST;
				lpV->Vertex.x = lpVertex->x;
				lpV->Vertex.y = lpVertex->y;
				lpV->Next = NULL;
				if(lpPrev)
					lpPrev->Next = lpV;
				else
					m_lpVertices = lpV;
				lpPrev = lpV;
			}
		}
		lpL = lpL->Next;
	}
}

VOID CMapConvert::BuildWadFile (LPSTR lpszMapName, LPSTR lpszFile, BOOL bDecorations)
{
	HANDLE hFile = CreateFile(lpszFile,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	if(hFile != INVALID_HANDLE_VALUE)
	{
		LPSTR lpszNames[] = {"DECORATE","SNDINFO",lpszMapName,"THINGS","LINEDEFS","SIDEDEFS","VERTEXES","SECTORS","BEHAVIOR","SCRIPTS"};
		LPSTR lpszName;
		DWORD dwTemp;
		LONG i, iStart, iSize, iOffset, iDir;
		HEADER Header;
		DIRECTORY Dir[10];
		CopyMemory(Header.ID,"PWAD",4);
		if(m_iConversionType == CTYPE_DOOM || m_iConversionType == CTYPE_EDGE)
		{
			Header.cLumps = 6;
			iStart = 2;
		}
		else if(m_iConversionType == CTYPE_ZDOOM)
		{
			Header.cLumps = 8;
			if(bDecorations)
			{
				Header.cLumps += 2;
				iStart = 0;
			}
			else
				iStart = 2;
		}
		else if (m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
		{
			//TJL - No extra lumps for the TC...
			Header.cLumps = 8;
			iStart = 2;
		}

		ZeroMemory(Dir,sizeof(DIRECTORY) * Header.cLumps);
		Header.iDir = 12;
		iOffset = sizeof(HEADER) + sizeof(DIRECTORY) * Header.cLumps;
		WriteFile(hFile,&Header,sizeof(HEADER),&dwTemp,NULL);
		// Serialize map data
		// ...and build directory
		SetFilePointer(hFile,iOffset,NULL,FILE_BEGIN);
		for(i = iStart; i < Header.cLumps + iStart; i++)
		{
			iDir = i - iStart;
			lpszName = lpszNames[i];
			CopyMemory(Dir[iDir].Name,lpszName,lstrlen(lpszName));
			switch(i)
			{
			case 0:	// ZDoom DECORATE
				if(m_iConversionType == CTYPE_ZDOOM && bDecorations)
					iSize = SerializeLump(hFile,"DECORATE.LMP");
				break;
			case 1:	// ZDoom SNDINFO
				if(m_iConversionType == CTYPE_ZDOOM && bDecorations)
					iSize = SerializeLump(hFile,"SNDINFO.LMP");
				break;
			case 2:	// MAP??
				iSize = 0;
				break;
			case 3:	// THINGS
				iSize = SerializeThings(hFile);
				break;
			case 4:	// LINEDEFS
				iSize = SerializeLinedefs(hFile);
				break;
			case 5:	// SIDEDEFS
				iSize = SerializeSidedefs(hFile);
				break;
			case 6:	// VERTEXES
				iSize = SerializeVertices(hFile);
				break;
			case 7:	// SECTORS
				iSize = SerializeSectors(hFile);
				break;
			case 8:	// BEHAVIOR
				if(m_iConversionType == CTYPE_ZDOOM)
					iSize = SerializeLump(hFile,"BEHAVIOR.LMP");
				else if (m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
					iSize = SerializeLump(hFile,"BEHAVIORTC.LMP");
				break;
			case 9:	// SCRIPTS
				if(m_iConversionType == CTYPE_ZDOOM)
					iSize = SerializeLump(hFile,"SCRIPTS.LMP");
				else if (m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
					iSize = SerializeLump(hFile,"SCRIPTSTC.LMP");
				break;
			}
			Dir[iDir].iOffset = iOffset;
			Dir[iDir].iSize = iSize;
			iOffset += iSize;
		}
		// Write directory
		SetFilePointer(hFile,12,NULL,FILE_BEGIN);
		WriteFile(hFile,Dir,sizeof(DIRECTORY) * Header.cLumps,&dwTemp,NULL);
		CloseHandle(hFile);
	}
}

ULONG CMapConvert::SerializeThings (HANDLE hFile)
{
	ULONG cSize = 0, dwWrite;
	LPTHING_LIST lpList = m_lpThings;
	while(lpList)
	{
		if(m_iConversionType == CTYPE_DOOM || m_iConversionType == CTYPE_EDGE)
		{
			THING1 Thing;
			lpList->lpThing->GetMapThing1(&Thing);
			WriteFile(hFile,&Thing,sizeof(THING1),&dwWrite,NULL);
		}
		else if(m_iConversionType == CTYPE_ZDOOM || m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
		{
			THING2 Thing;
			lpList->lpThing->GetMapThing2(&Thing);
			WriteFile(hFile,&Thing,sizeof(THING2),&dwWrite,NULL);
		}
		cSize += dwWrite;
		lpList = lpList->Next;
	}
	return cSize;
}

ULONG CMapConvert::SerializeLinedefs (HANDLE hFile)
{
	ULONG cSize = 0, dwWrite;
	LPLINE_LIST lpList = m_lpList;
	CMapLine* lpLine;
	CMapLinedef LineDef;
	SHORT sSideDef = 0;
	while(lpList)
	{
		lpLine = lpList->lpLine;
		LineDef.m_vFrom = FindVertex(&lpLine->m_vFrom);
		LineDef.m_vTo = FindVertex(&lpLine->m_vTo);
		LineDef.m_Flags = lpLine->m_Flags;
		if(m_iConversionType == CTYPE_DOOM || m_iConversionType == CTYPE_EDGE)
		{
			LineDef.m_Types = lpLine->m_Types;
			LineDef.m_Tag = lpLine->m_Tag;
		}
		else if(m_iConversionType == CTYPE_ZDOOM || m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
		{
			LineDef.m_Special = lpLine->m_Special;
			CopyMemory(LineDef.m_Args,lpLine->m_Args,5);
		}
		LineDef.m_sRight = sSideDef++;
		if(lpLine->m_Flags & 4)
			LineDef.m_sLeft = sSideDef++;
		else
			LineDef.m_sLeft = -1;
		if(m_iConversionType == CTYPE_DOOM || m_iConversionType == CTYPE_EDGE)
		{
			LINEDEF1 Linedef1;
			LineDef.GetMapLinedef1(&Linedef1);
			WriteFile(hFile,&Linedef1,sizeof(LINEDEF1),&dwWrite,NULL);
		}
		else if(m_iConversionType == CTYPE_ZDOOM || m_iConversionType == CTYPE_ZDOOMTC || m_iConversionType == CTYPE_BSTC)
		{
			LINEDEF2 Linedef2;
			LineDef.GetMapLinedef2(&Linedef2);
			WriteFile(hFile,&Linedef2,sizeof(LINEDEF2),&dwWrite,NULL);
		}
		cSize += dwWrite;
		lpList = lpList->Next;
	}
	return cSize;
}

ULONG CMapConvert::SerializeSidedefs (HANDLE hFile)
{
	ULONG cSize = 0, dwWrite;
	LPLINE_LIST lpList = m_lpList;
	CMapLine* lpLine;
	while(lpList)
	{
		lpLine = lpList->lpLine;
		WriteFile(hFile,&lpLine->m_sRight,sizeof(SIDEDEF),&dwWrite,NULL);
		cSize += dwWrite;
		if(lpLine->m_Flags & 4)
		{
			WriteFile(hFile,&lpLine->m_sLeft,sizeof(SIDEDEF),&dwWrite,NULL);
			cSize += dwWrite;
		}
		lpList = lpList->Next;
	}
	return cSize;
}

ULONG CMapConvert::SerializeVertices (HANDLE hFile)
{
	ULONG cSize = 0, dwWrite;
	LPVERTEX_LIST lpList = m_lpVertices;
	while(lpList)
	{
		WriteFile(hFile,&lpList->Vertex,sizeof(VERTEX),&dwWrite,NULL);
		cSize += dwWrite;
		lpList = lpList->Next;
	}
	return cSize;
}

ULONG CMapConvert::SerializeSectors (HANDLE hFile)
{
	ULONG cSize = 0, dwWrite;
	LPSECTOR_LIST lpList = m_lpSectors;
	while(lpList)
	{
		WriteFile(hFile,&lpList->Sector,sizeof(SECTOR),&dwWrite,NULL);
		cSize += dwWrite;
		lpList = lpList->Next;
	}
	return cSize;
}

ULONG CMapConvert::SerializeLump (HANDLE hFile, LPSTR lpszFile)
{
	ULONG iSize = 0;
	HANDLE hLump = CreateFile(lpszFile,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if(hLump == INVALID_HANDLE_VALUE)
	{
		CHAR szPath[_MAX_PATH], *lpszPtr;
		GetModuleFileName(NULL,szPath,_MAX_PATH);
		lpszPtr = strrchr(szPath,'\\');
		if(lpszPtr)
		{
			lstrcpy(lpszPtr+1,lpszFile);
			hLump = CreateFile(szPath,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		}
	}
	if(hLump != INVALID_HANDLE_VALUE)
	{
		DWORD dwTemp;
		LPBYTE lpData;
		iSize = GetFileSize(hLump,NULL);
		lpData = new BYTE[iSize];
		if(lpData)
		{
			ReadFile(hLump,lpData,iSize,&dwTemp,NULL);
			CloseHandle(hLump);
			WriteFile(hFile,lpData,iSize,&dwTemp,NULL);
			delete lpData;
		}
		else
		{
			CloseHandle(hLump);
			iSize = 0;
		}
	}
	return iSize;
}